* file appx_b.do for Using Stata for Principles of Econometrics, 5e

* Stata Do-file 
* copyright C 2018 by Lee C. Adkins and R. Carter Hill 
* used for "Using Stata for Principles of Econometrics, 5e" 
* by Lee C. Adkins and R. Carter Hill (2018)
* John Wiley and Sons, Inc.

* setup
version 15.1
clear all

/*---the binomial distribution---*/
* POE5 Appendix B.3.2

* binomial probabilities
scalar prob1 = binomial(13,7,0.7)
di "probability <= binomial(13,7,0.7) is " prob1

scalar prob2 = 1 - binomial(13,7,0.7)
di "probability > binomial(13,7,0.7) is " prob2

* binomial distribution
set obs 11
gen x = _n-1
gen p1 = binomialp(11,x,.3)     // probability
gen cp1 = binomial(11,x,.3)     // cumulative probability
gen p2 = binomialp(11,x,.5)
gen cp2 = binomial(11,x,.5)

list x p1 cp1 p2 cp2, separ(0) noobs

* plots
graph bar (asis) p1 p2, over(x)  ylabel(, nogrid)      ///
      legend( label(1 "p = .3") label(2 "p = .5") )    ///
	  ytitle( Probability of x successes )             ///
	  title(POE5 Figure B.3)                           ///
	  subtitle(Binomial Distribution: n=10)            ///
	  name(figb_3, replace)

/*---the poisson distribution---*/
* POE5 Appendix B.3.3
  
* poisson probabilities
clear all
set obs 11
gen x = _n-1
gen p1 = poissonp(3,x)      // mu=3 probability
gen cp1 = poisson(3,x)      // mu=3 cumulative probability
gen p2 = poissonp(4,x)
gen cp2 = poisson(4,x)

list x p1 cp1 p2 cp2, separ(0) noobs

* POE5 Figure B.4
graph bar (asis) p1 p2, over(x) ylabel(, nogrid)       ///
      legend( label(1 "mu = .3") label(2 "mu = .5") )  ///
      ytitle( Probability of x occurences )            ///
      title(POE5 Figure B.4)                           ///
      subtitle(Poisson distribution)                   ///
      name(figb_4, replace)

/*---the normal distribution---*/
* POE5 Appendix B.3.5

* plot standard normal density
twoway function y = normalden(x), range(-5 5) ///
       title("Standard normal density")       ///
       name(normal_pdf, replace)

* plot several normal densities 
twoway function y = normalden(x), range(-5 5)       ///
    || function y = normalden(x,0.8), range(-5 5)   ///
        lpattern(dash)                              ///
    || function y = normalden(x,1,0.8), range(-5 5) ///
        lpattern(dash_dot)                          ///
    ||, title("Normal Densities")                   ///
     legend(label(1 "N(0,1)") label(2 "N(0,0.8^2)") ///
     label(3 "N(1,0.8^2)")) name(normal_plots, replace)

* compute normal probabilities
scalar n_tail = normal(1.33)
di "lower tail probability N(0,1) < 1.33 is " n_tail

scalar prob = normal((6-3)/3) - normal((4-3)/3)
di "probability 3<=N(3,9)<=6 is " prob

* compute normal percentiles
scalar n_95 = invnormal(.95)
di "95th percentile of standard normal = " n_95

/*---the chi-square distribution---*/
* POE5 Appendix B.3.5

* Chi-square density
clear all

* POE5 Figure B.7(a)
twoway function y = chi2den(4,x), range(0 55) ylabel(,nogrid)    ///
  || function y = chi2den(10,x), range(0 55) lpattern(dash)      ///
  || function y = chi2den(30,x), range(0 55) lpattern(dash_dot)  ///
  ||, title(POE5 Figure B.7(a)) subtitle("Chi-square densities") ///
  legend(label(1 "df = 4") label(2 "df = 10")                    ///
  label(3 "df = 30")) name(figb7_a, replace) 

* chi-square probabilities
scalar chi2_tail = 1 - chi2(7,15)
di "upper tail probability chi2(7) > 15 is " chi2_tail

* chi-square critical values
scalar chi2_95 = invchi2tail(7,.05)
di "95th percentile of chi2(7) = " chi2_95

* POE5 Figure B.7(b) 
* noncentral chi-square density
twoway function y=nchi2den(10,0,x),range(0 40) lpattern(solid) ///
                ylabel(,nogrid)                               ///
 || function y=nchi2den(10,3,x),range(0 40) lpattern(dash)    ///
 || function y=nchi2den(10,6,x),range(0 40) lpattern(longdash_dot) ///
     , ytitle("Chi-square density function")                  ///
       xtitle("Chi-square value")                             ///
       title(POE5 Figure B.7(b))                              ///
       subtitle("Non-central chi-square , df=10, ncp=delta")  ///
       legend(label(1 "delta=0") label(2 "delta=3") 	      ///
       label(3 "delta=6")) name(noncentral_chisquare_dist, replace) 

/*---the t-distribution---*/
* POE5 Appendix B.3.7

* plot t(3)
clear all
twoway function y = normalden(x), range(-5 5)            ///
  || function y = tden(3,x), range(-5 5) lpattern(dash)  ///
  ||, subtitle("Standard normal and t(3)")               ///
  legend(label(1 "N(0,1)") label(2 "t(3)"))              ///
  title(POE5 Figure B.8(a)) name(figb8_a, replace) 

* t probabilities
scalar t_tail = ttail(3,1.33)
di "upper tail probability t(3) > 1.33 = " t_tail
di "lower tail probability t(3) < 1.33 = " 1 - ttail(3,1.33)

* t critical values
scalar t3_95 = invttail(3,0.05)
di "95th percentile of t(3) = " t3_95
di "5th percentile of t(3) = " invttail(3,0.95)

* t(38) shaded tail graphs
di "95th percentile of t(38) = " invttail(38,0.05)

* one-tail rejection region
twoway function y=tden(38,x), range(1.686 5) ///
       color(ltblue) recast(area)            ///
    || function y=tden(38,x), range(-5 5)    ///
       legend(off) plotregion(margin(zero))  ///
    ||, ytitle("f(t)") xtitle("t")           ///
        text(0 1.686 "1.686", place(s))      ///
        title("t(38) distribution")          ///
        subtitle("Right-tail rejection region") ///
        name(one_tail_reject, replace) 

* two-tail p-value
twoway function y=tden(38,x), range(1.9216 5)   ///
       color(ltblue) recast(area)               ///
    || function y=tden(38,x), range(-5 -1.9216) ///
       color(ltblue) recast(area)               ///
    || function y=tden(38,x), range(-5 5)       ///
    ||, legend(off) plotregion(margin(zero))    ///
        ytitle("f(t)") xtitle("t")              ///
        text(0 -1.921578 "-1.9216", place(s))   ///
        text(0 1.9216 "1.9216", place(s))       ///
        title("t(38) distribution")             ///
        subtitle("Pr|t(38)|>1.9216")            ///
        name(two_tail_p, replace)
		
* non-central t		
twoway function y=ntden(3,0,x), range(-4 8) lpattern(solid) ///
     ylabel(, nogrid)                                       ///
  || function y=ntden(3,1,x), range(-4 8) lpattern(dash)    ///
  || function y=ntden(3,2,x), range(-4 8) lpattern(longdash_dot) ///
   , ytitle("f(t)") xtitle("t") title(POE5 Figure B.8(b))   ///
     subtitle("Non-central t-distributions, m=3, ncp=delta") ///
     legend(label(1 "delta=0") label(2 "delta=1") ///
     label(3 "delta=2")) name(noncentral_t_dist, replace) 

/*---the F-distribution---*/
* POE5 Appendix B.3.8

* Plot F-density
twoway function y = Fden(8,20,x), range(0 6)   ///
       legend(off) plotregion(margin(zero))    ///
       ytitle("F-density") xtitle("x")         ///
       title("F(8,20) density")                ///
       name(F_density, replace)

* F probabilities
scalar f_tail = Ftail(8,20,3.0)
di "upper tail probability F(8,20) > 3.0 = " f_tail
di "upper tail probability F(8,20) > 3.0 = " 1-F(8,20,3.0)

* POE5 Figure B.9(a)
* F critical values
scalar f_95 = invFtail(8,20,.05)
di "95th percentile of F(8,20) = " f_95

twoway function y=Fden(8,20,x), range(2.447 5) ///
       color(ltblue) recast(area)              ///
    || function y=Fden(8,20,x), range(0 5)     ///
       legend(off) plotregion(margin(zero))    ///
    || , ytitle("f(F)") xtitle("F")            ///
       text(0 2.447 "2.447", place(s))         ///
       title("Figure B.9(a)")                  ///
       subtitle("F(8,20) 95th percentile")     /// 
       name(figb9_a, replace)

* POE5 Figure B.9(b)
* non-central F
twoway function y=nFden(8,20,0,x), range(0 5)      ///
                lpattern(solid) ylabel(, nogrid)   ///
  || function y=nFden(8,20,3,x), range(0 5)        ///
                lpattern(dash)                     ///
  || function y=nFden(8,20,6,x), range(0 5)        ///
                lpattern(longdash_dot)             ///
  || , ytitle("F-density") xtitle("F value")       ///
     subtitle("Non-central F(8,20,delta)")         ///
     title("POE5 Figure 8.9(b)")                   ///
     legend(label(1 "delta=0") label(2 "delta=3")  ///
     label(3 "delta=6"))                           ///
     name(noncentral_F_dist, replace) 

/*---the lognormal distribution---*/
* POE5 Appendix B.3.9
clear all
scalar sig1 = 1
scalar sig2 = 0.5
scalar sig3 = 0.25
scalar med = 1
scalar mu = ln(med)

* POE5 Figure B.10 
twoway function y = exp(-((1/(2*sig1^2))*(ln(x)-mu)^2))/          ///
                                         (x*sig1*sqrt(2*_pi)),    ///
                range(0 3) ylabel(, nogrid)                       ///
  ||   function y = exp(-((1/(2*sig2^2))*(ln(x)-mu)^2))/          ///
                                         (x*sig2*sqrt(2*_pi)),    ///
                range(0 3) lpattern(dash)                         ///
  ||   function y = exp(-((1/(2*sig3^2))*(ln(x)-mu)^2))/          ///
                                         (x*sig3*sqrt(2*_pi)),    ///
                range(0 3) lpattern(dash_dot)                     ///
     , ytitle(f(x)) title(POE5 Figure B.10)                       ///
       subtitle("Lognormal Densities: median = 1, sigma")         ///
       legend(label(1 "sigma=1") label(2 "sigma=0.5")             ///
       label(3 "sigma=0.25")) name(figb10, replace) 


/*---POE5 Example B.15---*/
* The Inversion Method: An Example

* generating triangular distribution random values

clear all
set obs 1000
set seed 12345
gen u1 = runiform()
set seed 1010101
label variable u1 "uniform random values"
histogram u1, bin(10) percent title(POE5 Figure B.13:uniform values) ///
   name(figb_13, replace)

gen y1 = sqrt(u1)

histogram y1, bin(10) percent                     ///
          title(POE5 Figure B.14:triangle values) ///
          name(figb_14, replace)

* generating extreme value distribution
clear
set obs 10000
set seed 12345
gen u1 = runiform()
gen v1 = -3+(_n-1)*13/10000
gen fev1 = exp(-v1)*exp(-exp(-v1))

twoway line fev1 v1, ytitle("Extreme value density")      ///
   title(POE5 Figure B.15) name(figb_15, replace)

* extreme value random values
gen ev1 = -log(-log(u1))
histogram ev1, bin(40) percent kdensity kdenopts(gaussian) ///
   title(POE5 Figure B.16) name(figb_16, replace)

/*---POE5 Example B.16---*/
* Linear Congruential Generator Example

* generating uniform random values
clear all
set obs 10001
gen double u1 = 1234567
gen double u2 = 987654321
scalar a = 1664525
scalar c = 1013904223
scalar m = 2^32
replace u1 = (a*u1[_n-1]+c) - m*ceil((a*u1[_n-1]+c)/m) + m if _n >1
replace u1 = u1/m

replace u2 = (a*u2[_n-1]+c) - m*ceil((a*u2[_n-1]+c)/m) + m if _n >1
replace u2 = u2/m

label variable u1 "uniform random number using seed = 1234567"
label variable u2 "uniform random number using seed = 987654321"

drop if _n==1
histogram u1, bin(20) percent title(POE5 Figure B.17) ///
          name(figb_17a, replace)
summarize u1

histogram u2, bin(20) percent name(figb_17b, replace)
summarize u2
