* file appx_c.do for Using Stata for Principles of Econometrics, 5e

* Stata Do-file 
* copyright C 2018 by Lee C. Adkins and R. Carter Hill 
* used for "Using Stata for Principles of Econometrics, 5e" 
* by Lee C. Adkins and R. Carter Hill (2018)
* John Wiley and Sons, Inc.

* setup
version 15.1
clear all

/*---POE5 Example C.1---*/
* Histogram of Hip Width Data
 
use stata/hip, clear
describe
histogram y, percent saving(hip_hist,replace)
graph export hip_hist.pdf, replace

* histogram using pull down menu
histogram y, width(1) start(13) percent

/*---POE5 Example C.2---*/
* Sample Mean of Hip Width Data
* summary statistics
summarize y, detail

* estimate mean
mean y

/*---POE5 Example C.3---*/
* Sampling Variation of Sample Means of Hip Width Data

* generate several normal variables
clear all
matrix C = (1, .5 \ .5, 1)
drawnorm x y, n(1000) corr(C) seed(12345)
summarize x y
tabstat x y, statistics (mean median variance semean)
corr x y
twoway scatter y x, saving(xynorm ,replace)

* Generate 10 samples and find the means
drawnorm x1 x2 x3 x4 x5 x6 x7 x8 x9 x10, seed(12345) ///
   n(50) means(17 17 17 17 17 17 17 17 17 17) clear
 
summarize
 
 tabstat x1 x2 x3, statistics( mean median semean var )
 
 /*---POE5 Example C.5---*/
 * Illustrating the Central Limit Theorem
 
clear all
set obs 1000
set seed 12345

* generate triangular distributed value
gen y1 = sqrt(runiform())
histogram y1, saving(triangle_hist ,replace)

* 11 more
forvalues rep=2/12 {
   gen y`rep' = sqrt(runiform())
   }

* standardize several and plot
foreach n in 3 7 12 {
   egen ybar`n' = rowmean(y1-y`n')
   gen z`n' = (ybar`n' - 2/3)/(sqrt((1/18)/`n'))
   histogram z`n', normal saving(ybar`n'_hist , replace)
   summarize z`n', detail
   }

/*---POE5 Example C.8---*/
* Simulating the Hip Data: Interval Estimates

use stata/table_c3, clear
summarize
scalar lb = r(mean) + invnormal(0.025)*sqrt(1/3)
scalar ub = r(mean) + invnormal(0.975)*sqrt(1/3)

* POE5 Table C.3

 di "95% CI centered at " r(mean) " is (" lb ", " ub ")"

   
/*---POE5 Example C.9---*/
* Simulating the Hip Data: Continued

* POE5 Table C.4

use stata/table_c4, clear 
summarize
forvalues n=1/10 {
    qui summarize y`n'
    scalar lb = r(mean) + invnormal(0.025)*sqrt(1/3)
    scalar ub = r(mean) + invnormal(0.975)*sqrt(1/3)
    di "95% CI centered at " r(mean) " is (" lb ", " ub ")"
    }

use stata/table_c4, clear
forvalues n=1/10 {
    qui summarize y`n'
    scalar se = r(sd)/sqrt(r(N))
    scalar lb = r(mean) - invttail(r(N)-1,0.025)*se
    scalar ub = r(mean) + invttail(r(N)-1,0.025)*se
    di "95% CI centered at " r(mean) " is (" lb ", " ub ")"
    }

/*---POE5 Example C.10---*/
* Interval Estimation Using the Hip Data

use stata/hip, clear

* automatic interval estimate
ci means y

* details of interval estimate
quietly summarize y, detail
return list
scalar ybar = r(mean)
scalar nobs = r(N)
scalar df = nobs - 1
scalar tc975 = invttail(df,.025)
scalar sighat = r(sd)
scalar se = sighat/sqrt(nobs)
scalar lb = ybar - tc975*se
scalar ub = ybar + tc975*se

di "95% CI centered at " r(mean) " is (" lb ", " ub ")"

* Section C.4.4
* Generating simulated data
clear all
set obs 30
set seed 12345
drawnorm x1-x10

* transform
forvalues n=1/10 {
        gen y`n' = 10 + sqrt(10)*x`n'
        }

* compute interval estimates
ci means y1-y10

/*---POE5 Example C.6---*/
* Sample Moments of the Hip Data
 
use stata/hip, clear

* Moments
quietly summarize y, detail
scalar nobs = r(N)
scalar ybar = r(mean)
scalar s2 = r(sd)^2
scalar list nobs ybar s2

scalar s_hat = r(sd)*sqrt((nobs-1)/nobs)
scalar var_ybar=s_hat^2/nobs
scalar sd_ybar=sqrt(var_ybar)
scalar list s_hat var_ybar sd_ybar

scalar mu_1 = r(mean)
scalar mu_2 = s_hat^2
scalar S = r(skewness)
scalar mu_3 = S*s_hat^3
scalar K = r(kurtosis)
scalar mu_4 = K*s_hat^4
scalar list mu_1 mu_2 mu_3 mu_4

* Using tabstat
tabstat y, statistics( mean sd semean skewness kurtosis )

/*---POE5 Example C.7---*/
* Using the Hip Data Estimates
 
scalar z=(18-ybar)/sqrt(s2)
scalar p = 1 - normal(z)
scalar list z p

scalar z = invnormal(.95)
scalar size_95 = sqrt(s2)*z+ybar
scalar list z size_95

/*---POE5 Example C.11---*/
* One-tail Test Using the Hip Data

********* hypothesis testing
* right tail test mu = 16.5
use stata/hip, clear
quietly summarize y, detail
scalar ybar = r(mean)
scalar nobs = r(N)
scalar df = nobs - 1
scalar sighat = r(sd)
scalar se = sighat/sqrt(nobs)
scalar t1 = (ybar - 16.5)/se
scalar tc95 = invttail(df,.05)

/*---POE5 Example C.13---*/
* One-tail Test p-value: The Hip Data

scalar p1 = ttail(df,t1)
di " Right Tail Test:" _n " t-statistic = "t1 _n ///
   " critical value = " tc95 _n" p-value = " p1 

* automatic version
ttest y==16.5

/*---POE5 Example C.12---*/
* Two-tail Test Using the Hip Data

* details
quietly summarize y, detail
scalar t2 = (ybar - 17)/se
scalar tc975 = invttail(df, .025)

/*---POE5 Example C.14---*/
* Two-Tail Test p-Value: The Hip Data

scalar p2 = 2*ttail(df,abs(t2))
di " Two Tail Test:" _n " t-statistic = "t2 _n ///
   " critical value = " tc975 _n" p-value = " p2 
di "two tail test"
di "tstat = " t2
di "tc975  = " tc975
di "pval  = " p2

* automatic version
ttest y==17

/*---POE5 Section C.7.1---*/
* Testing the Population Variance

* automatic test
sdtest y == 2

* details
quietly summarize y, detail
scalar s0 = 4
scalar sighat2 = r(Var)
scalar df = r(N)-1
scalar v = df*sighat2/s0
scalar chi2_95 = invchi2(df,.95)
scalar chi2_05 = invchi2(df,.05)
scalar p = 2*chi2(df,v)
di " Chi square test stat = " v _n ///
   " 5th percentile chisquare(49) = " chi2_05 _n  ///
   " 95th percentile chisquare(49) = " chi2_95 _n ///
   " 2 times p value = " p

/*---POE5 Section C.7.2---*/
* Testing the Equality of Two Population Means
 
clear all
drawnorm x1 x2, n(50) means(1 2) seed(12345)
summarize

* assuming variances are equal
ttest x1 == x2, unpaired

* assuming variances unequal
drawnorm x3 x4, n(50) means(1 2) sds(1 2) seed(12345)
ttest x3 == x4, unpaired unequal

/*---POE5 Section C.7.3---*/
* Testing the Ratio of Two Population Variances
 
sdtest x3 == x4

/*---POE5 Example C.15---*/
* Testing the Normality of the Hip Data

use stata/hip, clear

* Jarque_Bera test
* automatic test
sktest y

* details
quietly summarize y, detail
scalar nobs = r(N)
scalar s = r(skewness)
scalar k = r(kurtosis)
scalar jb = (nobs/6)*(s^2 + ((k-3)^2)/4)
scalar chi2_95 = invchi2(2,.95)
scalar pval = 1 - chi2(2,jb)
di "Jarqe-Bera test statistic = " jb _n ///
   "95th percentile chi2(2) = " chi2_95 _n ///
   "pvalue = " pval

/*---POE5 Example C.18---*/
* The "Wheel of Fortune" Game: Maximizing the Log-likelihood

* Likelihood
clear all
set obs 1000
gen p=.001*_n
gen l=2*ln(p)+ln(1-p)
twoway (line l p, sort)

sort l
list in -5/-1

/*---POE5 Example C.19---*/
* Estimating a Population Proportion

clear
set obs 200
scalar p = 75/200

/*---POE5 Example C.20---*/
* Testing a Population Proportion

scalar se_p = sqrt((p*(1-p))/_N)
scalar list se_p
scalar t = (p-0.4)/se_p
scalar pval = 2*ttail(_N-1,abs(t))
scalar crit = -invttail(_N-1,.025)
scalar list t pval crit

scalar lb = p - invttail(_N-1,.025)*se_p
scalar ub = p + invttail(_N-1,.025)*se_p
di "95% CI centered at " r(mean) " is (" lb ", " ub ")"

/*---POE5 Example C.21---*/
* Likelihood Ratio Test of the Population Proportion

scalar p = 75/200
scalar lnL_u = 75*ln(p)+(200-75)*ln(1-p)
scalar lnL_r = 75*ln(0.4)+(200-75)*ln(1-0.4)

scalar LR = 2*(lnL_u-lnL_r)
scalar pval = chi2tail(1,LR)
scalar crit = invchi2(1,.95)
scalar list lnL_u lnL_r LR pval crit

/*---POE5 Example C.22---*/
* Wald Test of the Population Proportion

scalar V = (p*(1-p))/_N
scalar W = (p-0.4)^2 * (1/V)
scalar pval = chi2tail(1,W)
scalar crit = invchi2(1,.95)
scalar list V W crit pval

/*---POE5 Example C.23---*/
* Lagrange Multiplier Test of the Population Proportion

scalar c = 0.4
scalar score = 75/c - (200-75)/(1-c)
scalar V = (c*(1-c))/_N
scalar LM = (score)^2 * V
scalar pval = chi2tail(1,LM)
scalar crit = invchi2(1,.95)
scalar list c score V LM pval crit

/*---POE5 Example C.24---*/
* Hip Data: Minimizing the Sum of Squares Function

use stata/hip, clear
summarize y
regress y

/* POE5 Appendix C.10---*/
* Kernel density estimator

use stata/kernel, clear

* Figure C.19
histogram x, freq width(.25) xlabel(2(1)12) ///
          saving(n1, replace)
histogram y, freq width(.25) xlabel(2(1)12)  ///
          saving(mix1,replace)
graph combine "n1" "mix1", cols(2) ysize(4) xsize(6) ///
   title("Figure C.19 Histograms of X and Y") saving(figc19,replace)

* Figure C.20
histogram x, freq width(.25) xlabel(2(1)12)  ///
        normal saving(n2, replace)
histogram y, freq width(.25) xlabel(2(1)12)  ///
        normal saving(mix2,replace)
graph combine "n2" "mix2", cols(2) ysize(4) xsize(6) ///
   title("Figure C.20 Normal Parametric Densities")  ///
   saving(figc20,replace)

* Figure C.21
histogram y, width(1) freq xlabel(2(1)12)   ///
        title("bin width=1") saving(y1,replace)

histogram y, width(.1) freq xlabel(2(1)12)  ///
        title("bin width=0.1") saving(y2,replace)

graph combine "y1" "y2", cols(2) ysize(4) xsize(6) ///
        title("Figure C.21 Different Bin Widths")  ///
        saving(figc21,replace)


* Figure C.22
histogram y, width(.25) freq xlabel(2(1)12)   ///
	kdensity kdenopts(gauss width(1.5)) title("bandwidth=1.5") ///
	saving(b1,replace)

histogram y, width(.25) freq xlabel(2(1)12)  ///
	kdensity kdenopts(gauss width(1)) title("bandwidth=1") ///
	saving(b2,replace)

histogram y, width(.25) freq xlabel(2(1)12)   ///
	kdensity kdenopts(gauss width(.4)) title("bandwidth=0.4") ///
	saving(b3,replace)

histogram y, width(.25) freq xlabel(2(1)12)   ///
	kdensity kdenopts(gauss width(.1)) title("bandwidth=0.1") ///
	saving(b4,replace)

graph combine "b1" "b2" "b3" "b4", cols(2) ysize(4) xsize(6) ///
	title("Figure C.22 Nonparametric Densities") ///
    saving(figc22,replace)



