* file chap01.do for Using Stata for Principles of Econometrics, 5e

* Stata do-file 
* copyright C 2018 by Lee C. Adkins and R. Carter Hill 
* used for "Using Stata for Principles of Econometrics, 5e" 
* by Lee C. Adkins and R. Carter Hill (2018)
* John Wiley and Sons, Inc.

*--------------------------------------------------------------
* this is a comment
*
/* this type of comment "/*  */" can contain other comments  */
*
*--------------------------------------------------------------

* setup
version 15.1                  
clear all                 // use this option if previous work can be erased
capture log close 

log using chap01, replace     // open log

* open data
use stata/cps5_small
describe

* assign or modify label
label variable wage "earnings per hour"

/*
 With few exceptions, the basic language syntax is

        [prefix :] command [varlist] [=exp] [if] [in] [weight]
                           [using filename] [, options]


    see                language element      description
    -------------------------------------------------------------------------
    help prefix        prefix :              prefix command
    help command       command               Stata command
    help varlist       varlist               variable list
    help exp           =exp                  expression
    help if            if                    if exp qualifier
    help in            in                    in range qualifier
    help weight        weight                weight
    help using         using filename        using filename modifier
    help options       options               options
    -------------------------------------------------------------------------
*/

* summarize and variations
help summarize
summarize					
summarize wage, detail		
summarize if exper >= 10	 
summarize in 1/50
summarize wage in 1/50, detail
summarize wage if female == 1 in 1/500, detail

*--------------------------------------------------------------
* path to summarize dialog box via pull-down menu
*
* Statistics > Summaries, tables, and tests > Summary and descriptive
*        statistics > Summary statistics
*
* or enter: db summarize
*
* or enter: help summarize and find Dialog at top right
*--------------------------------------------------------------

help comment

/* --------------------------------------------------------------

/// is a 'continuation comment'. It tells Stata to view the rest
    of the line as a comment 
    and to join the next line with the current line.

-----------------------------------------------------------------*/
* compact table of summary statistics
help tabstat
tabstat wage educ female,           /// list of variables                      
     statistics(n mean sum max min) /// list of statistics
     columns(variables)             //  put variables in columns                  

* histogram menu: Graphics > Histogram
help histogram
db histogram

histogram wage, percent             /// basic command
     title(Histogram of wage data)  /// figure title
     name(wage_hist, replace)       //  name figure   

graph save wage_hist "histogram of wages", replace

* enhanced histogram 
histogram wage, percent             /// basic command
     ytitle(Percent) xtitle(wage)   /// titles for axes
     title(Histogram of wage data)  /// figure title
     subtitle(cps5_small data)      /// figure subtitle
     name(wage_hist, replace)       
	 
histogram educ, discrete percent    /// basic command
     title(Histogram of educ data)  /// figure title
     subtitle(cps5_small data)      /// figure subtitle
     name(educ_hist, replace)  
     
histogram educ, discrete frequency  /// basic command
     title(Histogram of educ data)  /// figure title
     subtitle(cps5_small data)      /// figure subtitle
     name(educ_hist, replace)

* the tabulate command
tabulate educ        
              	 
* scatter diagram
twoway (scatter wage educ), name(wage_educ, replace)

* creating new variables
help generate

generate lwage = ln(wage)
label variable lwage "ln(wage)"

generate exper2 = exper^2
label variable exper2 "experience squared"

drop lwage exper2   // drop the variables

* Computing normal probabilities using scalars
help scalar
help functions
help normal

* an example
scalar phi = normal(1.27)  // normal cdf

* using display
display phi
di phi

display "Prob (Z <= 1.27) = " phi
di "Prob (Z <= 1.27) = " phi
di "Prob (Z <= 1.27) = " normal(1.27)

* Computing percentile values
scalar z90 = invnormal(.90)   // finding a percentile from N(0,1)
di "90th percentile value of standard normal is " z90

* Computing normal probabilities using local macros
tempname phi_temp z90_temp
scalar `phi_temp' = normal(1.27)
display `phi_temp'

scalar `z90_temp' = invnormal(.90)
di "90th percentile value of standard normal is " `z90_temp'

* Using temporary variables
tempvar expersq
gen `expersq' = exper^2
sum `expersq'

log close

* Use viewer to examine *.smcl log
help viewer
view chap01.smcl

* Use translate to convert *.smcl to text file *.log
translate chap01.smcl chap01.log, replace
