* file chap02.do for Using Stata for Principles of Econometrics, 5e

* Stata Do-file 
* copyright C 2018 by Lee C. Adkins and R. Carter Hill 
* used for "Using Stata for Principles of Econometrics, 5e" 
* by Lee C. Adkins and R. Carter Hill (2018)
* John Wiley and Sons, Inc.

* setup
version 15.1
clear all
capture log close

/*---Chapter 2.1 in POE5 Stata---*/    // This points to this manual’s sections
* open food data
log using chap02, replace
use stata/food, clear

/*---POE5 Example 2.3---*/             // This points to POE5 sections
* examine data
describe

* browse
list in 1/5
list food_exp in 1/5
list food_exp if income <= 10

list in 1/5, clean
list in 1/5, clean noobs

/*---Chapter 2.2 in POE5 Stata---*/
* compute summary statistics
summarize

* summarize food expenditure with detail
summarize food_exp, detail

/*---Chapter 2.3 in POE5 Stata---*/
* simple plot data
twoway (scatter food_exp income)
graph save food1, replace         // open for editing with: graph use food1

* save graph using saving
twoway (scatter food_exp income), saving(food1, replace)

* store the graph in memory only
twoway (scatter food_exp income), name(food1, replace)

/*---POE5 Figure 2.6---*/
* enhanced plot                            comments
twoway (scatter food_exp income), ///      basic plot control
        ylabel(0(100)600)         ///      Y axis 0 to 600 & ticks each 100
        xlabel(0(5)35)            ///      X axis 0 to 35 & ticks each 5 
        title(POE5 Figure 2.6)    ///      graph title
        subtitle(Food Expenditure Data) /// subtitle
        name(fig2_6, replace)           // graph name

help graph export

graph export food2.pdf, replace          // export graph  

/*---Chapter 2.4 in POE5 Stata---*/

/*---POE5 Example 2.4a---*/

* compute least squares regression
regress food_exp income
reg food_exp income

* calculate fitted values & residuals
predict yhat, xb
predict ehat, residuals

/*---POE5 Figure 2.8---*/
* plot fitted values and data scatter
twoway (scatter food_exp income) ///       basic plot control
       (lfit food_exp income),   ///       add linear fit
       ylabel(0(100)600)         ///       label Y axis
       xlabel(0(5)35)            ///       label X axis
       title(Figure 2.8 POE5)    ///       graph title
       subtitle(Fitted Regression Line) /// subtitle 
       name(fig2_8, replace)            // name graph

/*---Chapter 2.5 in POE5 Stata---*/

/*---POE5 Example 2.4b---*/

display _b[_cons] + _b[income]*20

scalar yhat20 = _b[_cons] + _b[income]*20
scalar list yhat20

estimates store model1
estimates restore model1

lincom _b[_cons] + _b[income]*20

margins, at( income=(10 20 30) )
margins, at( income=(10(10)40) ) vsquish

* using incomplete observations
edit
set obs 41
replace income = 20 in 41
predict yhat0
list income yhat0 in 41
drop in 41

* or simply use margins

margins, at(income=20)

/*---POE5 Example 2.4b---*/

* compute elasticity at means
margins, eyex(income) atmeans

* compute average of elasticities at each data point
margins, eyex(income)
generate elas = _b[income]*income/yhat
summarize elas

/*---Chapter 2.6 in POE5 Stata---*/

/*---POE5 Example 2.5---*/

* examine variances and covariances
estat vce

* to save changes to food data
save chap02.dta, replace

log close
translate chap02.smcl chap02.log, replace

/*---Chapter 2.7.1 in POE5 Stata---*/

/*---POE5 Example 2.6---*/

* new log file
log using chap02_quad, replace
clear all

* open br data and examine
use stata/br
describe
summarize

* create new variable
generate sqft2=sqft^2

* regression
regress price sqft2
predict priceq, xb

/*---POE5 Figure 2.14---*/

* plot fitted line
twoway (scatter price sqft)       ///   basic plot 
       (line priceq sqft,         ///   2nd plot: line is continuous 
       sort lwidth(medthick)),    ///   sort & change line thickness 
       name(fig2_14, replace)     //    name graph

* slope and elasticity calculations
di "slope at 2000 sqft = " 2*_b[sqft2]*2000
di "slope at 4000 sqft = " 2*_b[sqft2]*4000
di "slope at 6000 sqft = " 2*_b[sqft2]*6000

di "predicted price at 2000 sqft = " _b[_cons]+_b[sqft2]*2000^2
di "predicted price at 4000 sqft = " _b[_cons]+_b[sqft2]*4000^2
di "predicted price at 6000 sqft = " _b[_cons]+_b[sqft2]*6000^2

di "elasticity at 2000 = " 2*_b[sqft2]*2000^2/(_b[_cons]+_b[sqft2]*2000^2)
di "elasticity at 4000 = " 2*_b[sqft2]*4000^2/(_b[_cons]+_b[sqft2]*4000^2)
di "elasticity at 6000 = " 2*_b[sqft2]*6000^2/(_b[_cons]+_b[sqft2]*6000^2)

* using factor variables
regress price c.sqft#c.sqft
predict price2
margins, dydx(*) at(sqft=(2000 4000 6000))
margins, eyex(*) at(sqft=(2000 4000 6000))
margins, eyex(*)
regress, coeflegend

generate elas2 = 2*_b[c.sqft#c.sqft]*(sqft^2)/price2
summarize elas2

log close

/*---Chapter 2.7.2 in POE5 Stata---*/

log using chap02_llin, replace
clear all
use stata/br

/*---POE5 Figure 2.16---*/

* distribution of prices
summarize price, detail
histogram price, percent name(fig2_16a, replace)

* distribution of ln(price)
generate lprice = ln(price)
histogram lprice, percent name(fig2_16b, replace)

/*---POE5 Example 2.7---*/

* log-linear regression
reg lprice sqft
predict lpricef, xb

* price prediction using anti-log
generate pricef = exp(lpricef)

/*---POE5 Figure 2.17---*/
twoway (scatter price sqft)                          ///
       (line pricef sqft, sort lwidth(medthick)) ,   ///
       name(fig2_17, replace)

* predicted prices at given sqft
di "predicted price when sqft = 2000 is " exp(_b[_cons] + _b[sqft]*2000)
di "predicted price when sqft = 4000 is " exp(_b[_cons] + _b[sqft]*4000)
margins, at(sqft=(2000 4000)) expression(exp(xb())) vsquish

* slope at given price
di "slope at price $100000 = " _b[sqft]*100000
di "slope at price $500000 = " _b[sqft]*500000

* slope at predicted price given sqft
di "slope at 2000 sqft " _b[sqft]*exp(_b[_cons] + _b[sqft]*2000)
di "slope at 4000 sqft " _b[sqft]*exp(_b[_cons] + _b[sqft]*4000)
margins, at(sqft=(2000 4000)) expression(exp(xb())) dydx(sqft) vsquish

* elasticities at given sqft
di "elasticity at 2000 = " _b[sqft]*2000
di "elasticity at 4000 = " _b[sqft]*4000
margins, at(sqft=(2000 4000)) expression(exp(xb())) eyex(sqft) vsquish

* average marginal effects across sqft
generate me = _b[sqft]*exp(_b[_cons] + _b[sqft]*sqft)
summarize me
margins, expression(exp(xb())) dydx(sqft)

* average elasticities across sqft
generate elas = _b[sqft]*sqft
summarize elas
margins, expression(exp(xb())) eyex(sqft)


log close

/*---Chapter 2.8 in POE5 Stata---*/

* open new log
log using chap02_indicator, replace
clear all

* open utown data and examine
use stata/utown
describe
summarize

/*---POE5 Figure 2.18---*/

* histograms of utown data by neighborhood

histogram price if utown==0, width(12) start(130) percent  ///
          xtitle(House prices ($1000) in Golden Oaks)      ///
          xlabel(130(24)350) legend(off)                   ///
          name(not_utown_hist, replace)

histogram price if utown==1, width(12) start(130) percent  ///
          xtitle(House prices ($1000) in University Town)  ///
          xlabel(130(24)350) legend(off)                   ///
          name(utown_hist, replace)

* using by option
label define utownlabel 0 "Golden Oaks" 1 "University Town"
label value utown utownlabel
histogram price, by(utown, cols(1))            ///
          start(130) percent                   ///
          xtitle(House prices ($1000))         ///
          xlabel(130(24)350) legend(off)       ///
          name(utown_combine, replace)


* summary stats
summarize price if utown==0
summarize price if utown==1

* summary stats using by
by utown, sort: summarize price  

* summary stats using bysort
bysort utown: summarize price

* regression
regress price utown

* test of two means
ttest price, by(utown)
log close

/*---Appendix 2A in POE5 Stata---*/
/*---Average marginal effects---*/

/*---Appendix 2A.1 in POE5 Stata---*/
/*---Elasticity in a linear relationship---*/
* food expenditure example
log using chap02_food_me, replace 
clear all
use stata/food
summarize income
return list
scalar xbar = r(mean)
quietly regress food_exp income
margins, eyex(*) atmeans
nlcom _b[income]*xbar/(_b[_cons]+_b[income]*xbar) 
log close

/*---Appendix 2A.2 in POE5 Stata---*/
/*---Elasticity in a quadratic relationship---*/

log using chap02_quad_me, replace
clear all
use stata/br
quietly regress price c.sqft#c.sqft
margins, eyex(*) at(sqft=2000)
nlcom 2*_b[c.sqft#c.sqft]*(2000^2)/         ///
       (_b[_cons]+_b[c.sqft#c.sqft]*(2000^2))
log close

/*---Appendix 2A.3 in POE5 Stata---*/
/*---Slope in a log-linear model---*/
log using chap02_llin_me, replace 
use stata/br, clear
gen lprice = log(price)
quietly regress lprice sqft
nlcom _b[sqft]*exp(_b[_cons]+_b[sqft]*2000)

* using margins...
margins, at(sqft = 2000) expression(exp(xb())) dydx(sqft)
log close

/*---Appendix 2B.1 in POE5 Stata---*/
/*---The simulation experiment in POE5 Appendix 2H.4 & 2H.6---*/

* clear memory and start new log
clear all
log using chap02_app2H_fixed_x, replace 
version 13.1                     // you can change to your version

* define some global macros
global numobs 40                  // sample size  
global beta1 100                  // intercept parameter
global beta2 10                   // slope parameter
global sigma 50                   // error standard deviation

* random number seed
set seed 1234567

* create artificial data using y = beta1+beta2*x+e
set obs $numobs
generate x = 10
replace x = 20 if _n > $numobs/2
generate y = $beta1 + $beta2*x + rnormal(0,$sigma)

* regression with artifical data
regress y x
di "rmse " e(rmse)
estat vce
	
* data file mc1.data created using following command	
save mc1, replace

* program to generate data and estimate regression
program chap02sim, rclass
    version 13.1                        // Use version 13.1 to match POE5
    drop _all
    set obs $numobs
    generate x = 10
    replace x = 20 if _n > $numobs/2
    generate ey = $beta1 + $beta2*x
    generate e = rnormal(0, $sigma)
    generate y = ey + e
    regress y x
    return scalar b2 =_b[x]                        // saves slope
    return scalar b1 =_b[_cons]                    // saves intercept
    return scalar sig2 = (e(rmse))^2               // saves sigma^2
end

* simulate command
simulate b1r = r(b1) b2r=r(b2) sig2r=r(sig2) , ///
         reps(10000) nodots nolegend seed(1234567): chap02sim

* display experiment parameters
di " Simulation parameters"	
di " beta1 = " $beta1
di " beta2 = " $beta2
di " N = " $numobs
di " sigma^2 = " $sigma^2

* summarize experiment results
summarize, detail

* histogram sampling distribution of LS estimates
histogram b2r, percent normal name(ch2_b2_fix_x, replace)

*  Here is the example program that avoids globals
clear all

program chap02sim_alt, rclass
version 13.1                        // Use version 13.1 to match POE5
   syntax, [numobs(int 40) beta1(real 100) beta2(real 10) sigma(real 50)]
    drop _all
    set obs `numobs'
    generate x = 10
    replace x = 20 if _n > `numobs'/2
    generate ey = `beta1' + `beta2'*x
    generate e = rnormal(0, `sigma')
    generate y = ey + e
    regress y x
    return scalar b2 =_b[x]                        // saves slope
    return scalar b1 =_b[_cons]                    // saves intercept
    return scalar sig2 = (e(rmse))^2               // saves sigma^2
end

 simulate b1r = r(b1) b2r=r(b2) sig2r=r(sig2) , ///
         reps(10000) nodots nolegend seed(1234567): chap02sim_alt

log close

/*---Appendix 2B.2 in POE5 Stata---*/
/*---The simulation experiment in POE5 Appendix 2H.7---*/

version 13.1                // version 13.1 to match POE5 results

clear all
log using app2H-random_x, replace 
global numobs 40
global beta1 100
global beta2 10
global sigma 50

set seed 1234567

set obs $numobs
gen x = 15 + 1.6*rnormal()
gen y = $beta1 + $beta2*x + rnormal(0,$sigma)
regress y x

di "rmse " e(rmse)
estat vce

/* true var(b2) if sigma_x^2 = 2.56 */

scalar true_var_b2= (50^2)/(($numobs-3)*(2.56)) 
scalar true_sd_b2 = sqrt(true_var_b2)
di "true var b2 " true_var_b2
di "true sd b2 " true_sd_b2
	
save mc1_random_x, replace

program chap02sim, rclass
    version 13.1            // version 13.1 to match POE5 results
    drop _all
    set obs $numobs
    gen double x = 15 + 1.6*rnormal()
    gen double y = $beta1 + $beta2*x + rnormal(0,$sigma)
    regress y x
    return scalar b2 =_b[x]
    return scalar b1 =_b[_cons]
    return scalar sig2 = (e(rmse))^2
    return scalar b2se = _se[x]
end

simulate b1r = r(b1) b2r=r(b2) sig2r=r(sig2) b2ser=r(b2se) , ///
    reps(10000) seed(1234567) nodots nolegend: chap02sim

di " Simulation parameters"	
di " beta1    " $beta1
di " beta2    " $beta2
di " N        " $numobs	 
di " sigma^2  " $sigma^2
gen varb2 = b2ser^2

tabstat b1r b2r sig2r varb2 b2ser,   ///
    stat(mean var sd min max p1 p99) ///
    format(%10.4f)

histogram b2r, percent normal name(ch2_b2_ranx, replace)

log close

