* file chap03.do for Using Stata for Principles of Econometrics, 4e

* Stata Do-file 
* copyright C 2018 by Lee C. Adkins and R. Carter Hill 
* used for "Using Stata for Principles of Econometrics, 5e" 
* by Lee C. Adkins and R. Carter Hill (2018)
* John Wiley and Sons, Inc.

* setup
version 15.1
clear all
capture log close

/*---Chapter 3.1 in POE5 Stata---*/    // This points to this manual’s sections

* open food
use stata/food, clear

/*---POE5 Example 3.1---*/             // This points to POE5 sections

* estimate regression
reg food_exp income
reg food_exp income, level(90)

* compute t-critical value
scalar tc975 = invttail(38,.025)
di "t critical value 97.5 percentile = " tc975

* calculating 95% interval estimate
scalar ub2 = _b[income] + tc975*_se[income]
scalar lb2 = _b[income] - tc975*_se[income]
di "beta 2 95% interval estimate is " lb2 " , " ub2

* examples of computing t-critical values
di "t(30) 95th percentile = " invttail(30,0.05)
di "t(20) 95th percentile = " invttail(20,0.05)
di "t(20) 5th percentile  = " invttail(20,0.95)
di "t(30) 97.5th percentile = " invttail(30,0.025)
di "t(30) 2.5th percentile  = " invttail(30,0.975)

* using lincom
lincom income
lincom income, level(90)

/*---Chapter 3.2 in POE5 Stata---*/
/*---POE5 Example 3.2---*/    

* right-tail test ho:beta2 = 0
scalar tstat0 = _b[income]/_se[income]
di "t statistic for Ho: beta2=0 = " tstat0
di "t(38) 95th percentile = " invttail(38,0.05)

* using lincom
lincom income-5

/*---POE5 Example 3.3---*/    

* right-tail test ho:beta2 = 5.5
scalar tstat1 = (_b[income]-5.5)/_se[income]
di "t-statistic for Ho: beta2 = 5.5 is " tstat1
di "t(38) 99th percentile = " invttail(38,0.01)

* using lincom for calculation
lincom income-5.5

/*---POE5 Example 3.4---*/    

* left-tail test ho:beta2 = 15
scalar tstat2 = (_b[income]-15)/_se[income]
di "t-statistic for Ho: beta2 = 15 is " tstat2
di "t(38) 5th percentile = " invttail(38,0.95)
lincom income-15

/*---POE5 Example 3.5---*/    

* two-tail test ho:beta2 = 7.5
scalar tstat3 = (_b[income]-7.5)/_se[income]
di "t-statistic for Ho: beta2 = 7.5 is " tstat3
di "t(38) 97.5th percentile = " invttail(38,0.025)
di "t(38) 2.5th percentile = " invttail(38,0.975)
lincom income-7.5

/*---POE5 Example 3.6---*/    

* two-tail test ho:beta2 = 0
lincom income

* two-tail test ho:beta1 = 0
lincom _cons

/*---POE5 Example 3.3: continued---*/    

* p-value for right-tail test
scalar tstat1 = (_b[income]-5.5)/_se[income]
di "p-value for right-tail test ho:beta2 = 5.5 is " ttail(38,tstat1)

/*---POE5 Example 3.4: continued---*/    

* p-value for left-tail test
scalar tstat2 = (_b[income]-15)/_se[income]
di "p-value for left-tail test ho:beta2 = 15 is " 1-ttail(38,tstat2)

/*---POE5 Example 3.5: continued---*/    

* p-value for a two-tail test
scalar tstat3 = (_b[income]-7.5)/_se[income]
scalar phalf = ttail(38,abs(tstat3))
scalar p3 = 2*phalf
di "p-value for two-tail test ho:beta2 = 7.5 is " p3
di "p-value for ho:beta2 = 7.5 is " 2*ttail(38,abs(tstat3))

/*---POE5 Example 3.6: continued---*/    

* p-value for a two-tail test of signifiance
scalar tstat0 = _b[income]/_se[income]
scalar phalf = ttail(38,abs(tstat0))
scalar p0 = 2*phalf
di "p-value for two-tail test ho:beta2 = 0 is " p0

/*---POE5 Example3 3.7 & 3.8---*/    

* linear combinations of parameters
* estimating a linear combination
estat vce
lincom _cons + income*20

/*---POE5 Example 3.9---*/    

* testing a linear combination
lincom _cons + income*20 - 250


/*---Appendix 3A in POE5 Stata---*/
/*---Appendix 3A Graphing rejection regions---*/

clear all

* specify critcal values as globals
global t025=invttail(38,0.975)
global t975=invttail(38,0.025)

twoway (function y=tden(38,x), range(-5 $t025)    ///
                   color(ltblue) recast(area))    ///
       (function y=tden(38,x), range($t975 5)     ///
                      color(ltblue) recast(area)) ///
       (function y=tden(38,x), range(-5 5)),      ///
       legend(off) plotregion(margin(zero))       ///
                   ytitle("f(t)") xtitle("t")     ///
       xlabel(-5 $t025 0 $t975 5)                 ///
       name(twotail, replace)                     ///
       title("Two-tail rejection region" "t(38), alpha=0.05")

* one-tail rejection region
global t95 = invttail(38,0.05)
twoway (function y=tden(38,x), range($t95 5)      ///
                   color(ltblue) recast(area))    ///
       (function y=tden(38,x), range(-5 5)),      ///
       legend(off) plotregion(margin(zero))       ///
       ytitle("f(t)") xtitle("t")                 ///
       xlabel(-5  0 $t95 5)                       ///
       name(onetail, replace)                     ///
       title("Right-tail rejection region" "t(38), alpha=0.05")
  
/*---Appendix 3B in POE5 Stata---*/
/*---POE5 Chapter 3C.1 simulation---*/

* set up
clear all
version 13.1         // to match POE5 results

* define global variables
global numobs 40 
global beta1 100
global beta2 10
global sigma 50

* set random number seed
set seed 1234567

* generate sample of data
set obs $numobs
gen x = 10
replace x = 20 if _n > $numobs/2
gen y = $beta1 + $beta2*x + rnormal(0,$sigma)

* regression
regress y x

* test h0: beta2 = 10
scalar tstat = (_b[x]-$beta2)/_se[x]
di "ttest of H0: b2 = 10 t-value is " tstat

* program to generate data and to examine
* performance of interval estimator and
* hypothesis test  	
program chap03sim1, rclass
version 13.1                        // Use version 13.1 to match POE5
    syntax, [numobs(int 40) beta1(real 100) beta2(real 10) sigma(real 50)]
    drop _all
    set obs `numobs'
    generate x = 10
    replace x = 20 if _n > `numobs'/2
    generate ey = `beta1' + `beta2'*x
    generate e = rnormal(0, `sigma')
    generate y = ey + e
    regress y x
    return scalar b2 =_b[x]                       
    return scalar se2 =_se[x]                    
    end

simulate b2r = r(b2) se2r = r(se2),  ///
         reps(10000) nodots nolegend ///
         seed(1234567): chap03sim1

scalar tc975 = invttail($numobs-2,0.025)

* calculating 95% interval estimate
gen lb = b2r - tc975*se2r
gen ub = b2r + tc975*se2r
gen cover = ($beta2 >= lb) & ($beta2 <= ub)

* t-test
gen tstat = (b2r - $beta2)/se2r
scalar tc95 = invttail($numobs-2,0.05)
gen reject = tstat >= tc95

* examine some values when x is fixed
list b2r se2r tstat reject lb ub cover in 321/330, clean noobs

* summarize coverage and rejection when x is fixed
summarize cover reject

/*---POE5 Chapter 3C.4 simulation---*/
/*---Random x---*/

* set up
clear all
version 13.1

* define global variables
global numobs 40 
global beta1 100
global beta2 10
global sigma 50

* set random number seed
set seed 1234567

* generate sample of data
set obs $numobs
gen x = 15 + 1.6*rnormal()
gen y = $beta1 + $beta2*x + rnormal(0,$sigma)

* regression
regress y x

* test h0: beta2 = 10
scalar tstat = (_b[x]-$beta2)/_se[x]
di "ttest of ho b2 = 10 t-value is " tstat

* program to generate data and to examine
* performance of interval estimator and
* hypothesis test  	

program chap03sim2, rclass
version 13.1                        // Use version 13.1 to match POE5
    syntax, [numobs(int 40) beta1(real 100) beta2(real 10) sigma(real 50)]
    drop _all
    set obs `numobs'
    generate x = 15 + 1.6*rnormal()
    generate ey = `beta1' + `beta2'*x
    generate e = rnormal(0, `sigma')
    generate y = ey + e
    regress y x
    return scalar b2 =_b[x]                      
    return scalar se2 =_se[x]             
    end

simulate b2r = r(b2) se2r = r(se2),  ///
         reps(10000) nodots nolegend ///
         seed(1234567): chap03sim2


scalar tc975 = invttail($numobs-2,0.025)

* calculating 95% interval estimate
gen lb = b2r - tc975*se2r
gen ub = b2r + tc975*se2r
gen cover = ($beta2 >= lb) & ($beta2 <= ub)

* t-test
gen tstat = (b2r - $beta2)/se2r
scalar tc95 = invttail($numobs-2,0.05)
gen reject = tstat >= tc95

* examine some values when x is random
list b2r se2r tstat reject lb ub cover in 321/330, clean noobs

* summarize coverage and rejection when x is random
summarize cover reject


