* file chap05.do for Using Stata for Principles of Econometrics, 5e

* Stata Do-file
* copyright C 2018 by Lee C. Adkins and R. Carter Hill
* used for "Using Stata for Principles of Econometrics, 5e"
* by Lee C. Adkins and R. Carter Hill (2018)
* John Wiley and Sons, Inc.

* setup
version 15.1
capture log close
clear all

* open data
use stata/andy, clear

* Summary Statistics
summarize, detail

* List subset of observations
list in 1/5
list in -3/-1

/*---POE5 Example 5.2---*/ 
* Least squares regression with covariance matrix
regress sales price advert
estat vce

* Predict sales when price is 5.50 and adv is 1200
scalar yhat = _b[_cons] + _b[price]*5.50 + _b[advert]*1.2
scalar list yhat

* Using margins
margins, at(price = 5.50 advert = 1.2)

* Using lincom
lincom _cons + price*5.50 + advert*1.2

* The worst option, but it works
* Using the data editor to predict
use stata/andy, clear
regress sales price advert            // estimate a regression

clear                                 // open a new data set with 1 obs
set obs 1
gen price = 5.50                      // generate the independent obs 
gen advert = 1.2
predict yhat, xb                      // predict based on estimated model
list yhat price advert

* Rescale the variables
use stata/andy, clear
generate sales_star = sales * 1000
generate price_star = price * 100
regress sales_star price_star advert

/*---POE5 Example 5.2---*/ 
* Calculate sigma-hat square
regress sales price advert
ereturn list
scalar sighat2 = e(rss)/e(df_r)
scalar list sighat2

* Standard error of the regression
di sqrt(sighat2)

scalar sighat2_alt = e(rmse)^2
scalar list sighat2 sighat2_alt

/*---POE5 Example 5.4---*/ 
* Goodness-of-Fit
di "R-square " e(mss)/(e(mss)+e(rss))
di "R-square " 1-e(rss)/(e(mss)+e(rss)) 

* or
di "r2 = " e(r2)

* FWL 
use stata/andy, clear 
regress sales price advert
predict ehat, resid
di "Sum-of-Squared Errors from full regression: " e(rss)

quietly regress sales price
predict sales_resid, resid
quietly  regress advert price
predict advert_resid, resid
regress sales_resid advert_resid, noconst
predict double ehat_fwl, resid

list ehat_fwl ehat in 1/5
di "Sum-of-Squared Errors from FWL: " e(rss)

/*---POE5 Example 5.5---*/ 
* Variance, Covariance, and Standard Errors
quietly regress sales price advert
estat vce 

matrix VC_matrix = e(V)
matlist e(V), title(Variance-Covariance Matrix)

scalar var_b3 = VC_matrix[2,2]
scalar var_b2 = VC_matrix[1,1]
scalar var_b1 = VC_matrix[3,3]
scalar cov_b1_b2 = VC_matrix[3,1]
scalar cov_b1_b3 = VC_matrix[3,2]
scalar cov_b2_b3 = VC_matrix[1,2]
di "Variance b1=" var_b1 
di "Variance of b2=" var_b2
di "Variance of b3=" var_b3
di "Covariance of b1 and b2=" cov_b1_b2
di "Covariance of b1 and b3=" cov_b1_b3
di "Covariance of b2 and b3=" cov_b2_b3

/*---POE5 Example 5.6---*/ 
* Confidence Intervals
scalar bL = _b[price] - invttail(e(df_r),.025) * _se[price]
scalar bU = _b[price] + invttail(e(df_r),.025) * _se[price]
scalar list bL bU

scalar bL = _b[advert] - invttail(e(df_r),.025) * _se[advert]
scalar bU = _b[advert] + invttail(e(df_r),.025) * _se[advert]
scalar list bL bU

* Using the level() command to change size of default intervals
regress sales price advert, level(90)

/*---POE5 Example 5.7---*/ 
* Interval for a linear combination
* Easy way
lincom -0.4*price+0.8*advert, level(90)

* Hard way
matrix cov=e(V)
scalar lambda = -0.4*_b[price]+0.8*_b[advert]
scalar var_lambda = (-0.4)^2*cov[1,1]+(0.8)^2*cov[2,2]+2*(-0.4)*(0.8)*cov[1,2]
scalar se = sqrt(var_lambda)
scalar t = lambda/se
scalar lb = lambda-invttail(e(df_r),.05)*se
scalar ub = lambda+invttail(e(df_r),.05)*se
scalar list lambda var_lambda se t lb ub

/*---POE5 Example 5.8---*/ 
* t-ratios
scalar t1 = (_b[price]-0)/_se[price]
scalar p1 = 2*ttail(72,abs(t1))
scalar c1 = invttail(e(df_r),0.025)
if (abs(t1) > c1) {
 di abs(t1) " > " c1 ", Reject Ho" 
 }
 else {
 di "Do not Reject" 
 }

scalar list t1 p1 c1


/*---POE5 Example 5.9---*/ 
scalar t2 = (_b[advert]-0)/_se[advert]
scalar p2 = 2*ttail(72,abs(t1))
scalar c2 = invttail(e(df_r), 0.025)
if (abs(t2) > c2) {
 di abs(t2) " > " c2 ", Reject Ho" 
 }
 else {
 di "Do not Reject" 
 }
 scalar list t2 p2 c2

/*---POE5 Example 5.10---*/ 
* One sided significance test
scalar t1 = (_b[price]-0)/_se[price]
scalar crit = -invttail(e(df_r), 0.05)
scalar pval = 1-ttail(e(df_r), t1)
scalar list t1 crit pval

/*---POE5 Example 5.11---*/ 
* One sided test of Advertising effectiveness
scalar t2 = (_b[advert]-1)/_se[advert]
scalar crit = invttail(e(df_r), 0.05)
scalar pval = ttail(e(df_r), t2)
scalar list t2 crit pval

/*---POE5 Example 5.12---*/ 
* Linear combination
lincom -0.2*price-0.5*advert
scalar t = r(estimate)/r(se)
scalar crit = invttail(e(df_r),.05)
scalar pval = ttail(e(df_r),t)
scalar list crit t pval

return list

/*---POE5 Example 5.14---*/ 
* Polynomial
generate a2 = advert*advert
reg sales price advert a2
scalar me1 = _b[advert]+2*(.5)*_b[a2]
scalar me2 = _b[advert]+2*(2)*_b[a2]
scalar list me1 me2

* Polynomial using factor variables
regress sales price advert c.advert#c.advert
margins, dydx(advert) at(advert=(.5 2)) noatlegend

/*---POE5 Example 5.15---*/ 
* Interaction Variables 

use stata/cps5_small, clear
generate educ_exper = educ*exper
regress wage educ exper educ_exper
scalar me_8year = _b[exper]+_b[educ_exper]*8 
scalar me_16year = _b[exper]+_b[educ_exper]*16 
scalar me_20year = _b[exper]+_b[educ_exper]*20
 
display "Marginal effect of experience when education is 8 is " me_8year
display "Marginal effect of experience when education is 16 is" me_16year
display "Marginal effect of experience when education is 20 is " me_20year

* Using factor variables
regress wage educ exper c.educ#c.exper
margins, dydx(exper) at(educ=(8 16 20)) noatlegend
margins, dydx(exper) at(educ=(8 16 20)) level(90) noatlegend

/*---POE5 Example 5.16---*/ 
* Log wage specification
generate lwage = ln(wage)
regress lwage educ exper c.educ#c.exper c.exper#c.exper
reg lwage (c.educ c.exper)##c.exper
margins, dydx(exper) at(educ=(8 16) exper=(0 20)) vsquish
margins, dydx(educ) at(educ=(8 16) exper=(0 20)) vsquish

/*---POE5 Example 5.17---*/ 
* Nonlinear combinations of variables

use stata/andy, clear
qui regress sales price c.advert##c.advert
nlcom (1-_b[advert])/(2*_b[c.advert#c.advert])

/*---POE5 Example 5.18---*/
use stata/cps5_small, clear
gen lwage=ln(wage)
qui reg lwage (c.educ c.exper)##c.exper
nlcom (-_b[exper]-_b[c.educ#c.exper]*16)/(2*_b[c.exper#c.exper])

regress, coeflegend
scalar exper0 = (-_b[exper]-_b[c.educ#c.exper]*16)/(2*_b[c.exper#c.exper])
scalar list exper0

/*---POE5 Example 5.19---*/
* nonlinear function of one parameter
use stata/mc20, clear
reg y x
scalar df = e(df_r)
scalar g1hat = exp(_b[x]/10)
di "g1hat " g1hat

matrix V = e(V)
scalar v2 =V[1,1]
scalar vg1 = v2*(exp(_b[x]/10)*(1/10))^2
scalar seg1 = sqrt(vg1)
di "standard error of g1hat " seg1

scalar tc975 = invttail(df,.025)
scalar g1lb = g1hat-tc975*seg1
scalar g1ub = g1hat+tc975*seg1
scalar list g1hat seg1 tc975 g1lb g1ub

nlcom exp(_b[x]/10)
nlcom exp(_b[x]/10), df(18)

/*---POE5 Example 5.20---*/
* nonlinear function of two parameters

use stata/mc20, clear
reg y x
scalar df = e(df_r)
scalar g2hat = _b[_cons]/_b[x]
di "g2hat " g2hat

matrix V = e(V)
scalar v2 =V[1,1]
scalar v12 = V[1,2]
scalar v1=V[2,2]

scalar d1 = 1/_b[x]
scalar d2 = -_b[_cons]/(_b[x])^2

scalar vg2 = (d1^2)*v1 + (d2^2)*v2 + 2*d1*d2*v12
scalar seg2 = sqrt(vg2)
di "standard error of g2hat " seg2	

scalar tc975 = invttail(df,.025)
scalar g2lb = g2hat-tc975*seg2
scalar g2ub = g2hat+tc975*seg2
scalar list g2hat seg2 tc975 g2lb g2ub

nlcom _b[_cons]/_b[x]
nlcom _b[_cons]/_b[x], df(18)

/*---POE5 Appendix 5C---*/
* Monte Carlo experiment

version 13.1                                          // to match POE5 
clear all

capture program drop chap5sim
program chap5sim, rclass
    version 13.1
    args numobs b1 b2 s nu 
    tempvar x ey e y
    set obs `numobs'
    generate `x'=10
    replace `x' = 20 if _n > _N/2
    gen `ey' = `b1' + `b2' * `x'
    gen `e' = `s' * (rchi2(`nu')-`nu')/sqrt(2*`nu')
    gen `y' = `ey' + `e'
    summarize
    di `b1'
    qui regress `y' `x'

    di " Simulation parameters"
    di " Number of Observations " `numobs'
    di " beta1    " `b1'
    di " beta2    " `b2'
    di " N        " _N	 
    di " s  " `s'
 
    * the estimates
    return scalar b2 = _b[`x']
    return scalar b1 =_b[_cons]
    return scalar sig2 = (e(rmse))^2
    return scalar se2 = _se[`x'] 
	* the function g1
    nlcom exp(_b[`x']/10)
    matrix V1 = r(V)
    scalar vg1 = V1[1,1]
    return scalar seg1 = sqrt(vg1)                    // se(g1hat)
    * the function g2
    nlcom _b[_cons]/_b[`x']
    matrix V2 = r(V)
    scalar vg2 = V2[1,1]
    return scalar seg2 = sqrt(vg2)                    // se(g2hat)
 end 

simulate b1r=r(b1) b2r=r(b2) sig2r=r(sig2)            ///
         se2r = r(se2) seg1r=r(seg1) seg2r=r(seg2),   ///
         reps(10000) saving(mc, replace)              ///
         seed(1234567) nodots nolegend:               ///
		 chap5sim 40 100 10 50 4

use mc, clear
scalar df = 40 - 2 

* estimates Table 5C.1 for N = 40
scalar tc975 = invttail(df,.025)

* interval estimate of beta2
gen lbr = b2r - tc975*se2r
gen ubr = b2r + tc975*se2r

gen cover = (lbr < 10) & (10<ubr)                     // =1 if 10 in interval
gen tstat = (b2r-10)/se2r                             // tstat for true null
gen reject = (tstat > invttail(df,.05))               // = 1 if reject at 5%
gen close = (9 <= b2r) & (b2r <= 11)                  // = if b2 close to 10

summarize b1r b2r sig2r cover reject close, separator(0)
* delta method
* the estimates
gen g1hat = exp(b2r/10)
gen g2hat = b1r/b2r

* se(g1hat)
gen seg1 = se2r*exp(b2r/10)*.1                        // se(g1) calculation

list seg1 seg1r in 1/5, table                         // check calculation

* interval estimate
gen g1lb = g1hat-tc975*seg1
gen g1ub = g1hat+tc975*seg1

gen coverg1 = (g1lb <= exp(1)) & (exp(1)<=g1ub)       // =1 if true g1 inside
gen tg1 = (g1hat - exp(1))/seg1                       // tstat for true null
gen rejectg1 = (abs(tg1) > invttail(df,.025))         // = 1 if reject at 5%

* estimates Table 5C.2 for N = 40
summarize g1hat seg1 seg1r coverg1 tg1 rejectg1, separator(0)

* estimates Table 5C.3 for N = 40
summarize g2hat seg2r

* graphs
histogram b2r, percent normal title(Figure 5C.1 POE5)   ///
      subtitle(histogram of b2) name(fig5_c1, replace)
graph export fig5c_1.pdf, replace

histogram g1hat, percent normal title(Figure 5C.2 POE5) ///
      subtitle(histogram of g1) name(fig5_c2, replace)
graph export fig5c_2.pdf, replace

histogram g2hat, percent normal title(Figure 5C.3 POE5) ///
      subtitle(histogram of g2) name(fig5_c3, replace)
graph export fig5c_3.pdf, replace

save appx5c_mc_data_40, replace

/*---Appendix 5D Bootstrapping---*/

version 15.1
use stata/mc20, clear
regress y x, vce(boot, reps(100) seed(12345) nodots saving(bsdata,replace))

version 13.1                                               // to match POE5
* automatic bootstrap B=1999
bootstrap g1boot=exp(_b[x]/10) g2boot=_b[_cons]/_b[x],     ///
     nodots reps(1999) seed(12345) saving(g1data,replace) : reg y x
estat bootstrap, all

* Verify percentile interval for g1
use g1data, clear
qui summarize g1boot, detail
_pctile g1boot, p(2.5)
di " The 2.5% critical value is " r(r1) 

_pctile g1boot, p(97.5)
di " The 97.5% critical value is " r(r1) 

capture program drop refine
program refine, rclass
    version 13.1 
    reg y x
    nlcom exp(_b[x]/10)                          
    matrix g1 = r(b)
    scalar g1hat = g1[1,1]
    matrix vg1 = r(V)
    scalar varg1 = vg1[1,1]
    scalar seg1 = sqrt(varg1)
    return scalar tstar = (g1hat-$g1)/seg1       // tstat true in sample
end         

* Now do 1999 bootstrap replications
use stata/mc20, clear
quietly reg y x
quietly nlcom exp(_b[x]/10)
matrix g1 = r(b)
scalar g1hat = g1[1,1] 
global g1 = g1hat

bootstrap tstar=r(tstar), seed(12345) reps(1999) nodots /// 
     notable noheader saving(tstarg1, replace): refine

* Analyze the results to get the p-value
use tstarg1, clear
gen abt=abs(tstar)

_pctile abt, p(95)
di " The 97.5% critical value is " r(r1) 




