* file chap09.do for Using Stata for Principles of Econometrics, 5e

* cd ~/Documents/poe5         /// use to change working directory

* Stata do-file
* copyright C 2018 by Lee C. Adkins and R. Carter Hill
* used for "Using Stata for Principles of Econometrics, 5e"
* by Lee C. Adkins and R. Carter Hill (2018)
* John Wiley and Sons, Inc.

* setup
version 15.1                  
clear all                     /// use this option if previous work can be erased

* Model selection program
capture program drop modelselect
program define modelselect, rclass
  syntax varlist(numeric fv ts) [if] [in]
  quietly {
    local depvar:word 1 of `varlist'
    local indepvar:list varlist-depvar
    regress `varlist' `in'
    return scalar aic = ln(e(rss)/e(N))+2*e(rank)/e(N) 
    return scalar sc = ln(e(rss)/e(N))+e(rank)*ln(e(N))/e(N)
    return scalar R2 = e(r2)
	return scalar Adj_R2 = e(r2_a)
	}
end

* Confidence Interval
capture program drop ci_t
program define ci_t
 version 15.1
 args b se df a  //b=estimate, se=std error, df=d of f, a=alpha/2
 tempname fL fU
 scalar `fL' = `b' - invttail(`df',`a')*`se'
 scalar `fU' = `b' + invttail(`df',`a')*`se'
 display "The " 1-2*`a' " Confidence Interval is (" `fL' ", " `fU' ")"  
end

set obs 200
generate date = tq(1961q1) + _n-1
list date in 1/5
format %tq date
list date in 1/5
tsset date
save new.dta, replace

/*---POE5 Example 9.1---*/
* open log
use stata/usmacro, clear
gen date = qofd(dateid01)
list date in 1
format %tq date
tsset date
list date in 1

label var u "% Unemployment rate"
label var g "% GDP growth"

tsline u, lpattern(solid)
tsline g, lpattern(solid)

tsline u g, lpattern(solid dash)

list date u L.u D.u g L1.g L2.g L3.g in 1/5
list date u L.u D.u g L1.g L2.g L3.g in -5/-1

/*---POE5 Example 9.2---*/
ac u, lags(24)
corrgram u, lags(24)

/*---POE5 Example 9.3---*/
corrgram g, lags(5)

/*---POE5 Example 9.5---*/
* Estimating AR(2) model
reg u L.u L2.u
* Alternate forms of AR(2)
reg u L(1/2).u
reg L(0/2).u

/*---POE5 Example 9.6---*/
* Forecast Intervals for Unemplyment from AR(2)
* Forecasting using -arima- instead of -regress-
* which, of course, yields different predictions

use stata/usmacro, clear
gen date = qofd(dateid01)
format %tq date
tsset date

arima u in 3/273, ar(1/2)
tsappend, add(3)
predict uhat, xb        // for the point estimates
predict uhatse, mse     // for the standard error of prediction

reg u L(1/2).u 
scalar uhat1 = _b[_cons]+_b[L1.u]*u[273]+ _b[L2.u]*u[272]
scalar uhat2 = _b[_cons]+_b[L1.u]*uhat1+ _b[L2.u]*u[273]
scalar uhat3 = _b[_cons]+_b[L1.u]*uhat2+ _b[L2.u]*uhat1
scalar list uhat1 uhat2 uhat3

scalar var = e(rmse)^2
scalar se1 = sqrt(var)
scalar se2 = sqrt(var*(1+(_b[L1.u])^2))
scalar se3 = sqrt(var*((_b[L1.u]^2+_b[L2.u])^2+1+_b[L1.u]^2))
scalar list se1 se2 se3

ci_t uhat1 se1 e(df_r) 0.025
ci_t uhat2 se2 e(df_r) 0.025
ci_t uhat3 se3 e(df_r) 0.025

/*---POE5 Example 9.7---*/
use stata/usmacro, clear
gen date = qofd(dateid01)
format %tq date
tsset date
* Forecasting with and ARDL(2,1)
reg u L(1/2).u L.g
tsappend, add(3)
replace g = 0.869 in 274 
replace g = 1.069 in 275

scalar uhat1 = _b[_cons]+_b[L1.u]*u[273]+ _b[L2.u]*u[272] + _b[L1.g]*g[273]
scalar uhat2 = _b[_cons]+_b[L1.u]*uhat1+ _b[L2.u]*u[273] + _b[L1.g]*g[274]
scalar uhat3 = _b[_cons]+_b[L1.u]*uhat2+ _b[L2.u]*uhat1 + _b[L1.g]*g[275]
scalar list uhat1 uhat2 uhat3

scalar var = e(rmse)^2
scalar se1 = sqrt(var)
scalar se2 = sqrt(var*(1+(_b[L1.u])^2))
scalar se3 = sqrt(var*((_b[L1.u]^2+_b[L2.u])^2+1+_b[L1.u]^2))
scalar list se1 se2 se3

ci_t uhat1 se1 e(df_r) 0.025
ci_t uhat2 se2 e(df_r) 0.025
ci_t uhat3 se3 e(df_r) 0.025

use stata/usmacro, clear
generate date = qofd(dateid01)
format %tq date
tsset date
tsappend, add(3)
replace g = 0.869 in 274 
replace g = 1.069 in 275
replace g = 0 in 276

reg u L(1/2).u L.g
estimates store ardl21
forecast create mymodel, replace
forecast estimates ardl21
forecast solve

/*---POE5 Example 9.8---*/
reg L(0/2).u L.g
modelselect L(0/2).u L.g
return list

* AR model selection
use stata/usmacro, clear
gen date = qofd(dateid01)
format %tq date
tsset date

forvalues q=1/8 {
   forvalues p=1/8 {
         display "p=`p'  q=`q'"
      	  modelselect L(0/`p').u L(1/`q').g if date >= tq(1950q1)
          di "SC: " r(sc)
     	  }
   }


capture program drop ar_select
program ar_select, rclass
tempname select
postfile `select' p R2 Adj_R2 aic sc using results, replace
forvalues p=1/8 {
      	  modelselect L(0/`p').u  if date >= tq(1950q1)
		  post `select' (`p') (r(R2)) (r(Adj_R2)) (r(aic)) (r(sc)) 
     	  }
		  postclose `select'
		  end

ar_select 
use results, clear
sort aic
list in 1
sort sc
list in 1
sort Adj_R2
list in l		  

* ARDL model selection
use stata/usmacro, clear
gen date = qofd(dateid01)
format %tq date
tsset date

capture program drop ardl_select
program ardl_select, rclass
tempname select
postfile `select' p q R2 Adj_R2 aic sc using results, replace
forvalues q=1/8 {
   forvalues p=1/8 {
          modelselect L(0/`p').u L(1/`q').g if date >= tq(1950q1)
		  post `select' (`p') (`q') (r(R2)) (r(Adj_R2)) (r(aic)) (r(sc)) 
     	  }
  }
  postclose `select'
  end
  
ardl_select
use results, clear
sort aic
list in 1
sort sc
list in 1  
sort Adj_R2
list in l

/*---POE5 Example 9.9---*/
* Granger Causality
use stata/usmacro, clear
gen date = qofd(dateid01)
format %tq date
tsset date

reg L(0/2).u L.g
test L.g
      
reg L(0/2).u L(1/4).g
test (L1.g)(L2.g)(L3.g)(L4.g)
testparm L1.g L2.g L3.g L4.g

/*---POE5 Example 9.11---*/
reg L(0/2).u L.g
predict ehat21, res
ac ehat21, lags(24) generate(ac_u)
* approximate z scores
gen z=2/sqrt(e(N))
list ac_u z in 1/24
list ac_u if abs(ac_u)>z in 1/24

summarize g
scatter g L.g, xline(`r(mean)') yline(`r(mean)')
ac g, lags(12) generate(ac_g)

reg L(0/1).u L.g
predict ehat11, res
ac ehat11, lags(24)

/*---POE5 Example 9.12---*/
* LM tests
reg L(0/1).u L.g
estat bgodfrey, lags(1 2 3 4)

reg L(0/2).u L.g
estat bgodfrey, lags(1 2 3 4)   

/*---POE5 Example 9.13---*/
* Okun's Law
use stata/okun5_aus, clear
generate date = qofd(dateid01)
format %tq date
tsset date

tsline D.u, lpattern(solid)
tsline g, lpattern(solid)

reg D.u L(0/5).g
reg D.u L(0/4).g

scalar g0 = _b[g]
scalar g1 = g0+_b[L1.g]
scalar g2 = g1+_b[L2.g]
scalar g3 = g2+_b[L3.g]
scalar g4 = g3+_b[L4.g]

scalar growth = _b[_cons]/g4
scalar list g0 g1 g2 g3 g4 growth

/*---POE5 Example 9.14---*/
* A Phillips Curve
use stata/phillips5_aus, clear
generate date = qofd(dateid01)
format %tq date
tsset date
tsline inf, lpattern(solid)

reg inf du
regress inf du, vce(robust)
newey inf du, lag(4)

/*---POE5 Example 9.15---*/
* Phillips Curve with AR(1) errors
use stata/phillips5_aus, clear
generate date = qofd(dateid01)
format %tq date
tsset date

scalar B = round(4*(e(N)/100)^(2/9))
scalar list B

regress inf du
estimates store Wrong_SE
newey inf du, lag(4) 
estimates store HAC_4
esttab Wrong_SE HAC_4, compress se(%12.3f) b(%12.5f) gaps ///
       scalars(r2_a rss aic)            ///
       title("Dependent Variable: inf") ///
	   mtitles("LS" "HAC(4)")   

* --------------------------------------------------
* Nonlinear least squares of AR(1) regression model
* --------------------------------------------------

nl (inf = {b1}*(1-{rho}) + {b2}*du + {rho}*L.inf - {rho}*{b2}*(L.du)), /// 
         variables(inf du L.inf L.du)
* To see the coefficient legend use coeflegend option
nl, coeflegend
scalar delta = _b[b1:_cons]*(1-_b[rho:_cons])
scalar delta1 = - _b[rho:_cons]*_b[b2:_cons]
scalar list delta delta1
estimates store nlls

* AR(1) errors using FGLS
prais inf du, corc twostep
estimates store fgls

* AR(1) errors using arima
arima inf du, ar(1)
estimates store arima
esttab HAC_4 fgls arima nlls, compress se(%12.3f) b(%12.5f) gaps     ///
        scalars(rss rho) mtitle("OLS w/HAC" "2-step" "ARIMA" "NLLS") ///
		title("Dependent Variable: inf")
		
/*---POE5 Example 9.16---*/
* A Consumption Function
use stata/cons_inc, clear
generate date = qofd(dateid01)
format %tq date
tsset date

reg D.cons L.D.cons D.y
estat bgodfrey, lags(1/4)

scalar impact = _b[D.y]
scalar delay1 = _b[LD.cons]*_b[D.y]
scalar delay2 = (_b[LD.cons])^2*_b[D.y]
scalar total_Mult = _b[D.y]/(1-_b[LD.cons])
scalar list total_Mult impact delay1 delay2

* McLain & Wooldridge test
reg D.cons L.D.cons D.y
predict nuhat, resid
gen ehat = 0
replace ehat = _b[LD.cons]*L.ehat + nuhat in 3/227

reg nuhat L.D.cons D.y L.ehat
scalar stat = (e(N)-1)*e(r2)
scalar pval = chi2tail(1,stat) 
scalar list stat pval

/*---POE5 Examples 9.17 & 9.18---*/
* Multipliers for Infinite DL Model
use stata/okun5_aus, clear
generate date = qofd(dateid01)
format %tq date
tsset date

reg D.L(0/2).u L(0/1).g

* Exploiting variable creation
gen mult = _b[g] in 1
replace mult = L.mult*_b[L1.D.u]+_b[L1.g] in 2
replace mult = L.mult*_b[L1.D.u]+l2.mult*_b[L2.D.u] in 3/20
list mult in 1/12
* Graph the multipliers
gen lag = _n-1 in 1/12
line mult lag in 1/12
* Calculate the LR mulitplier and growth rate

qui summarize mult
di "Long run multiplier: " r(sum)
di "alpha: " _b[_cons]/(1 - _b[L1.D.u]-_b[L2.D.u])
di "Growth: " -_b[_cons]/(1 - _b[L1.D.u]-_b[L2.D.u])/r(sum)
drop mult lag

reg D.L(0/2).u L(0/1).g
predict uhat, resid
gen ehat = 0
replace ehat = _b[LD.u]*L.ehat + _b[L2D.u]*L2.ehat + uhat in 4/153

reg uhat D.L(1/2).u L(0/1).g L(1/2).ehat
scalar stat = (e(N)-3)*e(r2)
scalar pval = chi2tail(2,stat) 
scalar list stat pval

* Durbin Watson test
use stata/phillips5_aus, clear
generate date = qofd(dateid01)
format %tq date
tsset date

regress inf du
estat dwatson
