* file chap10.do for Using Stata for Principles of Econometrics, 5e

* Stata Do-file 
* copyright C 2018 by Lee C. Adkins and R. Carter Hill 
* used for "Using Stata for Principles of Econometrics, 5e" 
* by Lee C. Adkins and R. Carter Hill (2018)
* John Wiley and Sons, Inc.

* setup
version 15.1
capture log close
clear all

/*---POE5 Example 10.1---*/
* Least Squares Estimation of a Wage Equation

* open data and examine
use stata/mroz
describe
summarize 

* drop nonworking women and summarize
drop if lfp==0
summarize wage educ exper

* create variables
gen lwage = ln(wage)
gen exper2 = exper^2

* Least squares estimation
reg lwage educ exper exper2
estimates store ols

/*---POE5 Example 10.2---*/
* IV Estimation of a Simple Wage Equation
* Using only mothereduc as IV

regress lwage educ
pwcorr educ mothereduc
db ivregress
ivregress 2sls lwage (educ=mothereduc)
ivregress 2sls lwage (educ=mothereduc), small

* robust standard errors can be used
ivregress 2sls lwage (educ=mothereduc), vce(robust) small

/*---POE5 Example 10.3---*/
* 2SLS Estimation of a Simple Wage Equation

* first stage regression
reg educ mothereduc

* test IV strength
test mothereduc

* obtain predicted values
predict educhat

* 2sls using two OLS estimations
reg lwage educhat

/*---POE5 Example 10.4---*/
* Using Surplus Instruments in the Simple Wage Equation
* Add fathereduc as an IV
reg educ mothereduc fathereduc
test mothereduc fathereduc

reg educ mothereduc fathereduc, vce(robust)
test mothereduc fathereduc
predict educhat2
 
* 2sls using two OLS estimations
reg lwage educhat2

* IV estimation with surplus instruments
ivregress 2sls lwage (educ=mothereduc fathereduc), small 

/*---POE5 Example 10.5---*/
* IV/2SLS Estimation in the Wage Equation
reg educ exper exper2 mothereduc fathereduc

ivregress 2sls lwage exper exper2 (educ=mothereduc fathereduc), small 
estimates store iv

ivregress 2sls lwage exper exper2 (educ=mothereduc fathereduc) ///
         , small first

/*---POE5 Example 10.6---*/
* Checking Instrument Strength in the Wage Equation
estat firststage
 
ivregress 2sls lwage exper exper2 (educ=mothereduc fathereduc) ///
          , small vce(robust)
estat firststage

* Illustrate partial correlation
quietly ivregress 2sls lwage exper exper2 (educ=mothereduc), small 
estat firststage

* partial out exper and exper^2
reg educ exper exper2
predict reduc, resid

reg mothereduc exper exper2
predict rmom, resid

* partial correlation
pwcorr rmom reduc
return list
di "partial correlation squared = "r(rho)^2

* partial R^2
reg reduc rmom, noconstant

ivregress 2sls lwage exper exper2 (educ=mothereduc fathereduc), small
estat firststage
reg fathereduc exper exper2
predict rdad, resid

reg reduc rmom rdad, noconstant

/*---POE5 Example 10.7---*/
* Specification Tests for the Wage Equation
* Hausman test
reg educ exper exper2 mothereduc fathereduc
predict vhat, residuals
reg lwage exper exper2 educ vhat
reg lwage exper exper2 educ vhat, vce(robust)

* Hausman test automatic
hausman iv ols, constant sigmamore

* Testing surplus moment conditions
* obtain 2sls residuals
quietly ivregress 2sls lwage (educ=mothereduc fathereduc) exper exper2, small
predict ehat, residuals

* regress 2sls residuals on all IV
reg ehat exper exper2 mothereduc fathereduc
ereturn list

* NR^2 test
scalar nr2 = e(N)*e(r2)
scalar chic = invchi2tail(1,.05)
scalar pvalue = chi2tail(1,nr2)
di "R^2 from artificial regression = " e(r2)
di "NR^2 test of overidentifying restriction  = " nr2
di "Chi-square critical value 1 df, .05 level = " chic
di "p value for overidentifying test 1 df, .05 level = " pvalue

* Using estat
quietly ivregress 2sls lwage (educ=mothereduc fathereduc) exper exper2, small
estat overid

/*---POE5 Example 10.8---*/
* Testing for Weak Instruments
gen nwifeinc = (faminc-wage*hours)/1000

* 2SLS with various instrument sets

* Example 1: B=1, L=1
qui ivregress 2sls hours (mtr = exper) educ kidsl6 nwifeinc, small
estimates store m1
estat firststage

* first stage
qui reg mtr exper educ kidsl6 nwifeinc
estimates store r1
test exper

* Example 2: B=1, L=3
qui ivregress 2sls hours (mtr = exper exper2 largecity) ///
              educ kidsl6  nwifeinc, small
estimates store m2
estat firststage

* first stage
qui reg mtr exper exper2 largecity educ kidsl6  nwifeinc
estimates store r2
test exper exper2 largecity

* Example 3: B=2, L=2
qui ivregress 2sls hours (mtr educ =  mothereduc fathereduc) ///
              kidsl6 nwifeinc, small
estimates store m3
estat firststage

* first stage
qui reg mtr mothereduc fathereduc kidsl6 nwifeinc
estimates store r3a
test mothereduc fathereduc

* first stage
qui reg educ mothereduc fathereduc kidsl6 nwifeinc
estimates store r3b
test mothereduc fathereduc

* Example 4: B=2, L=3
qui ivregress 2sls hours (mtr educ = mothereduc fathereduc exper) ///
              kidsl6 nwifeinc, small
estimates store m4
estat firststage

* first stage
qui reg mtr mothereduc fathereduc exper kidsl6 nwifeinc
estimates store r4a
test mothereduc fathereduc exper

* first stage
qui reg educ mothereduc fathereduc exper kidsl6 nwifeinc
estimates store r4b
test mothereduc fathereduc exper

* create tables
esttab r1 r2 r3a r3b r4a r4b, compress t(%12.2f) b(%12.4f) nostar ///
       gaps title(POE5 Table 10A.3: First Stage Equations)        ///
       order(_cons exper educ kidsl6 nwifeinc exper2 largecity    ///
       mothereduc fathereduc)

esttab m1 m2 m3 m4, t(%12.2f) b(%12.4f) nostar        ///
       gaps title(POE5 Table 10A.4: IV estimations)   ///
       order(_cons mtr educ kidsl6 nwifeinc)

* Bonus material
* Appendix 10A Calculating Cragg-Donald Statistic

ivregress 2sls hours (mtr educ = mothereduc fathereduc exper) ///
          kidsl6 nwifeinc, small
estat firststage
scalar nobs = e(N)

foreach var of varlist mtr educ mothereduc fathereduc exper {
  regress `var' kidsl6 nwifeinc
  predict `var'r, resid
   }

* canonical correlations
canon (mtrr educr) (mothereducr fathereducr experr)
ereturn list
matrix r2=e(ccorr)
di "Calculation of Cragg-Donald statistic "
di "The canonical correlations "
matrix list r2
scalar mincc = r2[1,2]
di "The minimum canonical correlation = " mincc

scalar numiv = 6    // total number of IV = G + L
scalar L = 3        // number of external IV = L

scalar cd = ((nobs-numiv)/L)*(mincc^2)/((1-mincc^2))
di "The Cragg-Donald F-statistic = " cd

* Appendix 10B.1 Using Simulated Data

* open data
clear all
use stata/ch10, clear
summarize

* Least squares estimation
reg y x
estimates store ls

* IV estimation
reg x z1
predict xhat
reg y xhat

* IV estimation using automatic command
ivregress 2sls y (x=z1)
ivregress 2sls y (x=z1), small noheader
ivregress 2sls y (x=z2), small noheader
ivregress 2sls y (x=z3), small noheader

* IV estimation with surplus instruments
ivregress 2sls y (x=z1 z2), small noheader
estimates store iv

* Hausman test regression based
reg x z1 z2, noheader
predict vhat, residuals
reg y x vhat, noheader

* Hausman test automatic contrast
hausman iv ls, constant sigmamore

* Testing for weak instrument
reg x z1
reg x z2

* Joint test for weak instrument
reg x z1 z2
test z1 z2

* Testing for weak iv using estat
qui ivregress 2sls y (x=z1 z2), small
estat firststage

* Testing surplus moment conditions
predict ehat, residuals
reg ehat z1 z2
scalar nr2 = e(N)*e(r2)
scalar chic = invchi2tail(1,.05)
scalar pvalue = chi2tail(1,nr2)
di "NR^2 test of overidentifying restriction  = " nr2
di "Chi-square critical value 1 df, .05 level = " chic
di "p value for overidentifying test 1 df, .05 level = " pvalue

* Testing for weak iv using estat
quietly ivregress 2sls y (x=z1 z2), small
estat overid

* Testing surplus moment conditions
qui ivregress 2sls y (x=z1 z2 z3), small
predict ehat2, residuals
reg ehat2 z1 z2 z3
scalar nr2 = e(N)*e(r2)
scalar chic = invchi2tail(2,.05)
scalar pvalue = chi2tail(2,nr2)
di "NR^2 test of overidentifying restriction  = " nr2
di "Chi-square critical value 2 df, .05 level = " chic
di "p value for overidentifying test 2 df, .05 level = " pvalue

* Testing surplus moments using estat
quietly ivregress 2sls y (x=z1 z2 z3)
estat overid

* Appendix 10B.2 Sampling Properties of IV/2SLS

version 11.1                             // to match POE5 results
clear all

* specify constants to control simulation
*-----------------------------------------------------------------
global numobs 100   // number of simulated sample observations    
global pi     0.1   // reduced form parameter controls IV strength
global rho    0.8   // rho controls endogeneity
*-----------------------------------------------------------------

set obs $numobs
set seed 1234567    // random number seed

* correlation between e and v controls endogeneity
matrix sig = (1, $rho \ $rho, 1)                      // corr(e,v)
drawnorm e v, n($numobs) corr(sig)                    // e & v values

* create 3 uncorrelated standard normal variables
gen z1 = rnormal()
gen z2 = rnormal()
gen z3 = rnormal()
	
* DGP
generate x = $pi*z1 + $pi*z2 + $pi*z3 + v
generate y = x + e
pwcorr x e

* first stage regression using all IV
reg x z1 z2 z3

* OLS
reg y x

* 2sls
ivregress 2sls y (x=z1 z2 z3), small

* program used for simulation

program ch10sim, rclass
    version 11.1                         // to match POE5 results
    drop _all

    set obs $numobs
    matrix sig = (1, $rho \ $rho, 1)
    drawnorm e v, n($numobs) corr(sig)

    gen z1 = rnormal()
    gen z2 = rnormal()
    gen z3 = rnormal()

    * DGP
    generate x = $pi*z1 + $pi*z2 + $pi*z3 + v
    generate y = x + e                        // structural equation
    
    * first stage regression using all IV
    reg x z1 z2 z3
    return scalar rsq = e(r2)                // first stage r^2
    return scalar F=e(F)                     // first stage F
    predict vhat, r

    * OLS
    reg y x
    return scalar bols =_b[x]
    return scalar seols = _se[x]
    return scalar tols = (_b[x]-1)/_se[x]
    return scalar rols = abs(return(tols))>invttail($numobs-2,.025)

    * Hausman
    reg y x vhat
    return scalar haust = _b[vhat]/_se[vhat]
    return scalar haus = abs(return(haust))>invttail($numobs-3,.025)

* 2sls
    ivregress 2sls y (x=z1 z2 z3), small
    return scalar b2sls =_b[x]
    return scalar se2sls = _se[x]
    return scalar t2sls = (_b[x]-1)/_se[x]
    return scalar r2sls = abs(return(t2sls))>invttail($numobs-2,.025)
 end

simulate rsqf = r(rsq) Fr=r(F)                        ///   * first stage
         bolsr=r(bols) seolsr=r(seols) rolsr=r(rols)  ///   * ols
         b2slsr=r(b2sls) se2slsr=r(se2sls)            ///   * 2sls
         t2slsr=r(t2sls) r2slsr=r(r2sls)              ///   * t-test
         hausr=r(haus),                               ///   * hausman test
         reps(10000) nodots nolegend seed(1234567): ch10sim

di " Simulation parameters"	
di " rho = " $rho
di " N = " $numobs 
di " pi = " $pi
di " average first stage r-square" 
mean rsqf

di " average first stage F" 
mean Fr

* For each estimator compute 
* avg and standard deviation estimate beta
* avg nominal standard error
* avg percent rejection 5% test

di " OLS"
gen mseols = (bolsr-1)^2
tabstat bolsr seolsr rolsr mseols hausr, stat(mean sd)

di " 2sls"
gen mse2sls = (b2slsr-1)^2
tabstat b2slsr se2slsr r2slsr mse2sls, stat(mean sd)

