* file chap12.do for Using Stata for Principles of Econometrics, 5e

* Stata do-file
* copyright C 2018 by Lee C. Adkins and R. Carter Hill
* used for "Using Stata for Principles of Econometrics, 5e"
* by Lee C. Adkins and R. Carter Hill (2018)
* John Wiley and Sons, Inc.

* setup
version 15.1
clear all                     // use this option if previous work can be erased

* Utility to add logs to dataset
capture program drop makelog
program define makelog                              
   syntax varlist
   foreach var of local varlist {
      capture drop l_`var'
      gen l_`var' = ln(`var')                        // ln(x) = l_x 
      label variable l_`var' "ln(`var')"
      }
end

* Utility to add squares to data set
capture program drop makesquare
program define makesquare
   syntax varlist
   foreach var of local varlist {
      capture drop sq_`var'
      gen sq_`var' = `var'^2
      label variable sq_`var' "`var' squared"
      }
end

/*---POE5 Example 12.1---*/
* plot time series data

use stata/gdp5, clear
generate date = qofd(dateid01)
format date %tq
tsset date

qui twoway (tsline gdp), saving(gdp, replace)  
qui twoway (tsline D.gdp), saving(dgdp, replace) 
graph combine gdp.gph dgdp.gph, iscale(.5) xcommon

* Two ways to limit dates
summarize gdp D.gdp if date<=q(2000q3)
summarize gdp D.gdp if date>q(2000q3)

summarize gdp D.gdp if tin(,2000q3)
summarize gdp D.gdp if tin(2000q4,) 	

ac gdp, lags(24) 
ac D.gdp, lags(24)

* ---------------------------------------
* Extract dates with year and quarter 
* ---------------------------------------

use stata/gdp5, clear
generate date = qofd(dateid01)
format date %tq
tsset date

gen double newdate = dofq(date)
gen y = year(newdate)
gen q = quarter(newdate)

list date y q in 1/9

* ---------------------------------------
* Graph time-series 
* Graphs are named with replace option
* and combined.
* ---------------------------------------

use stata/usdata5, clear
generate date = mofd(dateid01)
format date %tm
tsset date

qui twoway (tsline infn), saving(infn, replace)
qui twoway (tsline D.infn), saving(dinfn, replace) yline(0)
qui twoway (tsline ffr), saving(f, replace)
qui twoway (tsline D.ffr), saving(df, replace) yline(0)
qui twoway (tsline br), saving(b, replace)
qui twoway (tsline D.br), saving(db, replace) yline(0)

graph combine   gdp.gph dgdp.gph infn.gph  ///
                dinfn.gph f.gph df.gph    ///
                b.gph db.gph , cols(2) iscale(.5)

summarize if tin(1954m8,1985m10)
summarize if tin(1985m11,2016m12) 

* To get summary stats for all variables and differences without generate
summarize infn ffr br D.infn D.ffr D.br if tin(1954m8,1985m10)
summarize infn ffr br D.infn D.ffr D.br if tin(1985m11,2016m12) 

/*---POE5 Example 12.2---*/
* A Deterministic Trend for Wheat Yield

use stata/toody5, clear
format year %ty
tsset year

makelog y
label variable l_y "ln(yield)"

use stata/toody5, clear
makelog y
tsset year

reg l_y year
predict l_yhat
twoway line l_y l_yhat year

reg rain year
predict rain_hat
twoway line rain rain_hat year

reg l_y t rain c.rain#c.rain
scalar se_rain = _se[rain]

makesquare rain
reg l_y t
predict e_ly, resid
reg rain t
predict e_rain, resid
reg sq_rain t
predict e_rain2, resid

reg l_y t rain c.rain#c.rain
scalar se_rain = _se[rain]
reg e_ly e_rain e_rain2, noconst
scalar se_e_rain = sqrt(46/44)*_se[e_rain]
scalar list se_rain se_e_rain

/*---POE5 Example 12.3---*/
* Spurious Regression

use stata/spurious, clear
gen time = _n
tsset time

regress rw1 rw2
estat bgodfrey

tsline rw1 rw2, name(g1, replace)
scatter rw1 rw2, name(g2, replace)

regress rw1 rw2
estat bgodfrey

/*---POE5 Example 12.4---*/
* Checking the Two Interest Rate Series for Stationarity
use stata/usdata5, clear
generate date = mofd(dateid01)
format date %tm
tsset date

reg D.ffr L.ffr D.L(1/2).ffr 
reg D.br L.br D.L(1/2).br 

dfuller ffr, lags(2) regress
dfuller br, lags(2) regress

/*---POE5 Example 12.5---*/
*Is GDP Trend Stationary?

use stata/gdp5, clear
generate date = qofd(dateid01)
format date %tq
tsset date
gen t = _n

tsline gdp

reg D.gdp t L.gdp D.L(1/2).gdp
dfuller gdp, trend lags(2) regress

/*---POE5 Example 12.6---*/
*Is Wheat Yield Trend Stationary?

use stata/toody5, clear
makelog y
tsset year

reg D.l_y t L.l_y 
dfuller l_y, trend lags(0) regress

* ----------------------
* Other Unit root tests 
* ----------------------

use stata/usdata5, clear
generate date = mofd(dateid01)
format date %tm
tsset date

* DF-GLS tests
dfgls ffr
dfgls br

tsline ffr
tsline br

* Phillips-Perron tests
pperron ffr, regress 
pperron br, regress 

/*---POE5 Example 12.7---*/
* Cointegration

dfuller D.ffr, lags(1) noconstant regress
dfuller D.br, lags(1) noconstant regress

/*---POE5 Example 12.8---*/
* Engle Granger cointegrations test

regress br ffr
predict ehat, residual
regress D.ehat L.ehat L.D.ehat l2.D.ehat, noconstant

* Using the built-in Stata commands
dfuller ehat, noconstant lags(2)
drop ehat

/*---POE5 Example 12.9---*/
* Error correction model - nlls

gen Db=D.br
nl (Db = -{alpha}*(L.br-{beta1}-{beta2}*L.ffr)+  /// 
    {gamma1}*L.Db + {gamma2}*L2.Db +             ///   
    {delta0}*D.ffr  + {delta1}*D.L.ffr +         ///
    {delta2}*D.L2.ffr + {delta3}*D.L3.ffr +      /// 
    {delta4}*D.L4.ffr),                          /// 
    variables(L.br L.Db L2.Db L.ffr D.L.ffr D.L2.ffr D.L3.ffr D.L4.ffr)

display "Theta parameter: " 1-_b[alpha:_cons]

gen ehat = L.br - _b[beta1:_cons]-_b[beta2:_cons]*L.ffr
reg D.ehat L.ehat L.D.ehat L2.D.ehat, noconst
di "t-ratio for the lagged residual: " _b[L.ehat]/_se[L.ehat]

/*---POE5 Example 12.10---*/
* No cointegration--First Differences

use stata/cons_inc, clear
generate date = qofd(dateid01)
format %tq date
tsset date

tsline cons, tline(1985q1) name(g1, replace)
tsline y, tline(1985q1) name(g2, replace)
reg D.cons t L.cons L.D.cons if tin(1985q1,)

dfuller cons if tin(1985q1,), lags(1) trend regress
dfuller y if tin(1985q1,), lags(1) trend regress

reg cons t y if tin(1985q1,)
predict ehat_c, resid

dfuller ehat_c if tin(1985q2,), lags(1) regress noconst
reg D.ehat_c L.ehat_c D.L.ehat_c if tin(1985q2,), noconst

dfuller D.cons if tin(1985q1,), regress
dfuller D.y if tin(1985q1,), regress

reg D.cons D.y L.D.cons if tin(1985q1,)
