* file chap13.do for Using Stata for Principles of Econometrics, 5e

* Stata do-file
* copyright C 2018 by Lee C. Adkins and R. Carter Hill
* used for "Using Stata for Principles of Econometrics, 5e"
* by Lee C. Adkins and R. Carter Hill (2018)
* John Wiley and Sons, Inc.

* setup
version 15.1
clear all

/*---POE5 Example 13.1---*/
* VECM for GDP

use stata/gdp, clear
gen date = tq(1970q1) + _n - 1 
format %tq date
tsset date

* Plot the series to identify constants 
* and trends.

tsline aus usa, scheme(s2gcolor) name(level, replace)
tsline D.aus D.usa, scheme(s2gcolor) name(difference, replace)

* Test for Unit Roots
* Experiment with noconst, trend, drift, and lag length
dfuller aus, regress lags(1)
dfuller usa, regress lags(3)

* Cointegrating regression
reg aus usa, noconst
predict ehat, res
tsline ehat, name(C1, replace)

* Engle-Granger Test for Cointegration
reg D.ehat L.ehat, noconst
dfuller ehat, lags(0) noconst

* VECM 

regress D.aus L.ehat
regress D.usa L.ehat
drop ehat

/*---POE5 Example 13.2---*/
* VAR model for consumption and income

use stata/fred5, clear
gen date = qofd(dateid01) 
format %tq date
tsset date

* Plot the series to identify constants 
* and trends

tsline consn y, legend(lab (1 "ln(RPCE)") lab(2 "ln(RPDI)")) /// 
       name(l1, replace) 
tsline D.consn D.y, legend(lab (1 "ln(RPCE)") lab(2 "ln(RPDI)")) ///
       name(d1, replace) 

* Stationarity Analysis
* Brute force, 1 equation at a time

qui reg L(0/1).D.consn L.consn 
di "Lags = 1"   
estat bgodfrey, lags(1 2 3)
qui reg L(0/2).D.consn L.consn 
di "Lags = 2"  
estat bgodfrey, lags(1 2 3)
qui reg L(0/3).D.consn L.consn 
di "Lags = 3"
estat bgodfrey, lags(1 2 3)
dfuller consn, lags(3)
dfuller consn, lags(3) trend

* Use the loop to compute stats for y
forvalues p = 1/3 {
   qui reg L(0/`p').D.y L.y
   di "Lags =" `p'  
   estat bgodfrey, lags(1 2 3)
}
dfuller y, lags(1)
dfuller y, lags(1) trend

* Cointegration Test: Case 2
reg consn y
predict ehat, res
reg D.ehat L.ehat D.L.ehat, noconst
di _b[L.ehat]/_se[L.ehat]

reg D.consn D.L.consn D.L.y
reg D.y D.L.consn D.L.y

varbasic D.consn D.y, lags(1/1) step(12) nograph

* Test residuals for autocorrelation

varlmar 

* Try extending lags to 3 and repeat
quietly varbasic D.consn D.y, lags(1/4) step(12)
varlmar

* There is evidence of autocorrelation so extend the lag to 3

* Selecting lags using model selection criteria
varsoc D.consn D.y, maxlag(5)

* Impulse responses and variance decompositions
qui varbasic D.consn D.y, lags(1/1) step(12)
irf table irf
irf table fevd

irf graph irf, name(g1, replace)
irf graph fevd, name(g2, replace)

* Combining irf and fevd in a single table
irf table irf fevd, title("Combined IRF/FEVD for C and Y")


