* file chap14.do for Using Stata for Principles of Econometrics, 5e

* Stata do-file
* copyright C 2018 by Lee C. Adkins and R. Carter Hill
* used for "Using Stata for Principles of Econometrics, 5e"
* by Lee C. Adkins and R. Carter Hill (2018)
* John Wiley and Sons, Inc.

* setup
version 15.1
clear all

/*---POE5 Example 14.1---*/
* Characteristics of Financial Variables

use stata/returns5, clear

* ----------------------------------------------
* Create dates and declare time series 
* ----------------------------------------------

gen date = mofd(dateid01)
format date %tm
tsset date

* ----------------------------------------------
* Time series plots and histograms
* ----------------------------------------------

qui tsline nasdaq, name(nas, replace)
qui tsline allords, name(a, replace)
qui tsline ftse, name(f, replace)
qui tsline nikkei, name(nk, replace)
graph combine nas a f nk, cols(2) name(all1, replace)

qui histogram nasdaq, normal name(nas, replace)
qui histogram allords, normal name(a, replace)
qui histogram ftse, normal name(f, replace)
qui histogram nikkei, normal name(nk, replace)
graph combine nas a f nk, cols(2) name(all2, replace)

/*---POE5 Example 14.2---*/
* Simulating Time-Varying Volatility

clear
global nobs = 200
set obs $nobs
gen time = _n
tsset time

set seed 101010101
gen e_arch=0
gen e = rnormal(0,1)
replace e_arch=e*(1 + .8*(L1.e_arch)^2)^.5 if time>=2 & time<=$nobs

gen y = e
gen y_arch = e_arch 

tsline y, name(g7, replace)
tsline y_arch, name(g8, replace)                   
graph combine g7 g8, cols(2) name(arch, replace)

arch y, arch(1)
arch y_arch, arch(1)

histogram y, normal name(g9, replace)
histogram y_arch, normal name(g10, replace)
graph combine g9 g10, cols(2) name(arch_hist, replace)

/*---POE5 Example 14.3---*/
* Testing for ARCH in BrightenYourDay (BYD) Lighting

* Load byd, create dates and declare time series

use stata/byd, clear
gen time = _n
tsset time

tsline r, name(g1, replace)

* ----------------------------------------------
* LM test for ARCH(1)
* ----------------------------------------------

regress r
predict ehat, residual

gen ehat2 = ehat * ehat
reg ehat2 L.ehat2
scalar TR2 = e(N)*e(r2)
scalar pvalue = chi2tail(1,TR2)
scalar crit = invchi2tail(1,.05)
scalar list TR2 pvalue crit

* ----------------------------------------------
* Built-in LM Test for ARCH(1)
* ----------------------------------------------

regress r
estat archlm, lags(1)

/*---POE5 Example 14.4---*/
* ARCH Model Estimates for BrightenYourDay (BYD) Lighting

* ARCH(1)

arch r, arch(1)

/*---POE5 Example 14.5---*/
* Forecasting BrightenYourDay (BYD) Volatility

predict htarch1, variance
tsline htarch, name(g2, replace)

gen ht_1 = _b[ARCH:_cons]+_b[ARCH:L1.arch]*(L.r-_b[r:_cons])^2
list htarch ht_1 in 496/500

/*---POE5 Example 14.6---*/
* A GARCH Model for BrightenYourDay

* GARCH(1,1)

arch r, arch(1) garch(1)
predict htgarch, variance
tsline htgarch, name(g3, replace)

/*---POE5 Example 14.7---*/
* A T-GARCH Model for BYD
 
* Threshold GARCH

arch r, arch(1) garch(1) tarch(1)
predict httgarch, variance
tsline httgarch, name(g4, replace)

/*---POE5 Example 14.8---*/
* A GARCH-in-Mean Model for BYD
 
* GARCH in mean

arch r, archm arch(1) garch(1) tarch(1)
predict m_mgarch, xb
predict htmgarch, variance
qui tsline m_mgarch, name(g5, replace)
qui tsline htmgarch, name(g6, replace)
graph combine g5 g6, cols(1)

summarize m_mgarch r, detail 
histogram m_mgarch, normal

