* file chap15.do for Using Stata for Principles of Econometrics, 5e

* Stata Do-file 
* copyright C 2018 by Lee C. Adkins and R. Carter Hill 
* used for "Using Stata for Principles of Econometrics, 5e" 
* by Lee C. Adkins and R. Carter Hill (2018)
* John Wiley and Sons, Inc.

* setup
version 15.1
capture log close
clear all

/*---POE5 Example 15.1---*/
* A Microeconomic Panel

* Open and examine the data

use stata/nls_panel
describe
xtset id year
xtdescribe

summarize lwage educ south black union exper tenure
list id year lwage educ south black union exper tenure ///
     in 1/10, noobs sepby(id)

/*---POE5 Example 15.2---*/
* Using T = 2 Differenced Observations for a Production Function

clear all
use stata/chemical2
describe

list firm year lsales lcapital llabor in 1/6, sepby(firm) noobs

* sort the data
sort firm year
keep if year > 2004
global v1list lsales lcapital llabor
foreach var of global v1list {
        by firm: gen `var'd = `var'[_n] - `var'[_n-1]
        }

* OLS
reg lsales lcapital llabor, noheader

* Differenced
reg lsalesd lcapitald llabord, noconstant noheader

* Using time series operators
clear all
use stata/chemical2
keep if year > 2004
xtset firm year
regress d.(lsales lcapital llabor), noconstant 

/*---POE5 Example 15.3---*/
* Using T = 2 Differenced Observations for a Wage Equation
clear all
use stata/nls_panel
keep if year > 86
xtset id year

global vlist lwage educ exper

foreach var of global vlist {
        by id: gen `var'd = `var'[_n] - `var'[_n-1]
        }

reg lwaged educd experd, noconstant noheader
summarize educd

reg lwaged experd, noconstant noheader

* Using time series operator
clear all
use stata/nls_panel
keep if year > 86
xtset id year
regress d.(lwage educ exper), noconstant

/*---POE5 Example 15.4---*/
* Using the Within Transformation with T = 2 Observations for
* a Production Function

clear all
use stata/chemical2
keep if year >2004
sort firm year
global v1list lsales lcapital llabor
foreach var of global v1list {
        by firm: egen `var'm = mean(`var')
        gen `var'w = `var' - `var'm
        by firm: gen `var'd = `var'[_n] - `var'[_n-1]
        }
	
list firm year lsales lsalesd lsalesm lsalesw in 1/4, sepby(firm) noobs

reg lsalesw lcapitalw llaborw, noconstant
estimates store within
scalar cf = sqrt(398/198)
di "cf " cf
scalar se2 = cf*_se[lcapitalw]
scalar se3 = cf*_se[llaborw]
scalar list se2 se3

xtset firm year
xtreg lsales lcapital llabor, fe

/*---POE5 Example 15.5---*/
* Using the Within Transformation with T = 3 Observations for a
* Production Function

clear all
use stata/chemical2
sort firm year
global v1list lsales lcapital llabor
foreach var of global v1list {
        by firm: egen `var'm = mean(`var')
        gen `var'w = `var' - `var'm
        by firm: gen `var'd = `var'[_n] - `var'[_n-1]
        }
	
list firm year lsales lsalesd lsalesm lsalesw in 1/6, sepby(firm) noobs

reg lsalesw lcapitalw llaborw, noconstant
scalar cf = sqrt(598/398)
di "cf " cf
scalar se2 = cf*_se[lcapitalw]
scalar se3 = cf*_se[llaborw]
scalar list se2 se3

xtset firm year
xtreg lsales lcapital llabor, fe

/*---POE5 Example 15.6---*/
* Using the Fixed Effects Estimator with T = 3 Observations for
* a Production Function

tab firm, gen(d)

* LSDV-unrestricted model
reg lsales lcapital llabor d1-d200, noconstant	
reg lsales lcapital llabor bn.firm, noconstant

xtset firm year
xtreg lsales lcapital llabor, fe

* within-unrestricted model
reg lsalesw lcapitalw llaborw, noconstant

* ols-restricted model
reg lsales lcapital llabor

/*---POE5 Example 15.7---*/
* Using Pooled OLS with Cluster-Robust Standard Errors
* for a Production Function

reg lsales lcapital llabor
estimates store m1
reg lsales lcapital llabor, vce(robust)
estimates store m2
reg lsales lcapital llabor, vce(cluster firm)
estimates store m3

esttab m1 m2 m3, b(4) se(4) wide compress nostar replace ///
    mtitles("regular" "robust" "cluster")	
esttab m1 m2 m3, b(4) t(4) wide compress nostar replace	 ///
    mtitles("regular" "robust" "cluster")
 
/*---POE5 Example 15.8---*/
* Using Fixed Effects and Cluster-Robust Standard Errors
* for a Production Function

xtreg lsales lcapital llabor, fe
estimates store xt1
xtreg lsales lcapital llabor, fe vce(robust)
estimates store xt2
xtreg lsales lcapital llabor, fe vce(cluster firm)
estimates store xt3

esttab xt1 xt2 xt3, b(4) se(4) wide compress nostar replace ///
    mtitles("regular" "robust" "cluster")	
esttab xt1 xt2 xt3, b(4) t(4) wide compress nostar replace	///
    mtitles("regular" "robust" "cluster")


/*---POE5 Example 15.9---*/
* Random Effects Estimation of a Production Function

xtreg lsales lcapital llabor, re
xtreg lsales lcapital llabor, re vce(cluster firm)

*---POE5 Example 15.10---*/
* Random Effects Estimation of a Wage Equation
clear all
use stata/nls_panel
xtset id year

global x1list educ exper exper2 tenure tenure2 black south union

xtreg lwage $x1list, fe
estimates store fe

xtreg lwage $x1list, re theta
estimates store re

* Calculation of RE transformation parameter
xtreg lwage $x1list, fe
scalar sig2e = e(sigma_e)^2

* Automatic Between estimator
xtreg lwage $x1list, be 
ereturn list

* Save sigma2_between and compute theta
scalar sig2b = e(rss)/e(df_r)
scalar sig2u = sig2b - sig2e/e(Tbar)
scalar sigu = sqrt(sig2u)
scalar theta = 1-sqrt(sig2e/(e(Tbar)*sig2u+sig2e))
di "Components of variance"
di "sig2e   = " sig2e " variance of overall error e(it)"
di "sige    = " sqrt(sig2e) " standard deviation of e(it)"
di "sig2b   = " sig2b " variance from between regression "
di "sig2u   = " sig2u " derived variance mu(i) "
di "sigu    = " sigu  " standard deviation mu(i) "
di "theta   = " theta " transformation parameter "

* transform data including intercept
gen one = 1
xtset
global v2list lwage one $x1list

foreach var of global v2list {
        by i: egen `var'bar = mean(`var')
        gen `var'd = `var' - theta*`var'bar
        }

* RE is ols applied to transformed data
reg lwaged educd blackd experd exper2d tenured tenure2d ///
           southd uniond oned, noconstant

*---POE5 Example 15.11---*/
* Testing for Random Effects in a Production Function
clear all
use stata/chemical3
xtset firm year

* re
xtreg lsales lcapital llabor, re
xttest0

* ols
reg lsales lcapital llabor
scalar sse = e(rss)
predict ehat, resid
by firm:egen sum_ehat_id = total(ehat)
gen sum_ehat_id_sq = sum_ehat_id^2

quietly xtdescribe
scalar t = r(mean)
scalar n = r(N)
qui summarize sum_ehat_id_sq, meanonly
scalar sumtop = r(N)*r(mean)/t
scalar lmstat = ((sumtop/sse)-1)*sqrt(n*t/(2*(t-1)))
scalar lmstat_sq = lmstat^2
scalar list sse t n sumtop lmstat lmstat_sq

*---POE5 Example 15.12---*/
* Testing for Endogenous Random Effects
* in a Production Function

* Hausman contrast test
qui xtreg lsales lcapital llabor, fe
estimates store fe
scalar b2_fe = _b[lcapital]
scalar se2_fe = _se[lcapital]

qui xtreg lsales lcapital llabor, re 
estimates store re
scalar b2_re = _b[lcapital]
scalar se2_re = _se[lcapital]

scalar t2_num = b2_fe - b2_re
scalar t2_den = sqrt(se2_fe^2 - se2_re^2)
scalar t2 = t2_num/t2_den
scalar list b2_fe b2_re t2_num
scalar list se2_fe se2_re t2_den
scalar list t2

hausman fe re

*---POE5 Example 15.13---*/
* Testing for Endogenous Random Effects in a Wage Equation
clear all
use stata/nls_panel
xtset id year

global x1list educ exper exper2 tenure tenure2 black south union

xtreg lwage $x1list, fe
estimates store fe

xtreg lwage $x1list, re 
estimates store re

hausman fe re

*---POE5 Example 15.14---*/
* The Mundlak Approach for a Production Function

clear all
use stata/chemical3
sort firm year, stable
xtset firm year

global v2list lcapital llabor

foreach var of global v2list {
        by firm: egen `var'bar = mean(`var')
        }

reg lsales lcapital llabor lcapitalbar llaborbar, vce(cluster firm)
test lcapitalbar llaborbar 
scalar chisq = r(df)*r(F)
scalar list chisq

xtreg lsales lcapital llabor lcapitalbar llaborbar, re
test lcapitalbar llaborbar 

xtreg lsales lcapital llabor lcapitalbar llaborbar, re vce(cluster firm)
test lcapitalbar llaborbar 

*---POE5 Example 15.15---*/
* The Mundlak Approach for a Wage Equation
clear all
use stata/nls_panel
xtset id year

global x1list exper exper2 tenure tenure2 south union

foreach var of global x1list {
        by id: egen `var'bar = mean(`var')
        }

* Regression based Hausman test
global xlist3 *bar $x1list educ black

xtreg lwage $xlist3, re 
testparm *bar

* Hausman test with robust VCE
xtreg lwage $xlist3, re vce(cluster id)
testparm *bar

* Add year specific indicator variable
tabulate year, generate(d)
xtreg lwage $xlist3 d2-d5, re vce(cluster id)
testparm *bar

* using factor variables
xtreg lwage $xlist3 i.year, re vce(cluster id)
testparm *bar

*---POE5 Example 15.16---*/
* The Hausman–Taylor Estimator for a Wage Equation
clear all
use stata/nls_panel
xtset id year
global x1list exper exper2 tenure tenure2 south union
global x2list educ black $x1list

xthtaylor lwage $x2list, endog(south educ) constant(educ black)


