* file chap16.do for Using Stata for Principles of Econometrics, 5e

* Stata Do-file 
* copyright C 2018 by Lee C. Adkins and R. Carter Hill 
* used for "Using Stata for Principles of Econometrics, 5e" 
* by Lee C. Adkins and R. Carter Hill (2018)
* John Wiley and Sons, Inc.

* setup
version 15.1
capture log close
clear all

/*---POE5 Example 16.2---*/
* A Transportation Problem: The Linear Probability Model

* examine data
use stata/transport
describe
summarize

* linear probability model
regress auto dtime, vce(robust)
lincom _cons + dtime
predict phat
gen yhat = (phat >= 0.5)
label variable yhat "model prediction"
label define busauto 0 "bus" 1 "auto"
label values yhat busauto
label variable auto "transportation"
label values auto busauto

tabulate auto yhat
list auto bustime autotime dtime phat yhat if (phat < 0) | (phat > 1)

/*---POE5 Example 16.3---*/
clear

input y x
  1 1.5
  1 0.6
  0 0.7
end   
list y x

probit y x

/*---POE5 Example 16.4---*/
* The Transportation Data: Probit

clear all
use stata/transport
probit auto dtime

* probability auto given dtime=0
nlcom normal(_b[_cons])

* beta1 + beta2*2
lincom _b[_cons]+_b[dtime]*2 

* standard normal density
nlcom (normalden(_b[_cons]+_b[dtime]*2))

* marginal effect when dtime=2
nlcom (normalden(_b[_cons]+_b[dtime]*2)*_b[dtime])

* direct calculation of predicted probability at dtime=3
nlcom (normal(_b[_cons]+_b[dtime]*3) )

/*---POE5 Example 16.5---*/
* The Transportation Data: More Postestimation Analysis

* MER
nlcom (normalden(_b[_cons]+_b[dtime]*2)*_b[dtime])

* MEM
summarize dtime
scalar xbar = r(mean)
nlcom (normalden(_b[_cons]+_b[dtime]*xbar)*_b[dtime])

* AME
* marginal effect evaluated at each observation
gen ame = normalden(_b[_cons]+_b[dtime]*dtime)*_b[dtime]

* average marginal effect
tabstat ame, stat(n mean sd min max)

* ame at means
margins, dydx(dtime) atmeans

* average marginal effects
margins, dydx(dtime)

* predicted probability at dtime = 3
margins, at(dtime=3)

* 95% interval estimate of predicted probability at dtime = 3
margins, at(dtime=3) df(19)

/*---Appendix 16A---*/
* probit details

* probit
qui probit auto dtime
ereturn list

* list e(V)
matlist e(V)

* ME at dtime=2
margins, dydx(dtime) at(dtime=2)

* dg-dbeta1
nlcom (-normalden(_b[_cons]+_b[dtime]*2)* ///
                 (_b[_cons]+_b[dtime]*2)*_b[dtime])

* dg-dbeta2
nlcom (normalden(_b[_cons]+_b[dtime]*2)*  ///
             (1-(_b[_cons]+_b[dtime]*2)*_b[dtime]*2))

* average marginal effects
margins, dydx(dtime)
margins, dydx(dtime) df(19)

* dg2-dbeta1
gen dg21 = -normalden(_b[_cons]+_b[dtime]*dtime)* ///
          (_b[_cons]+_b[dtime]*dtime)*_b[dtime] 

gen dg22 = normalden(_b[_cons]+_b[dtime]*dtime)* ///
          (1-(_b[_cons]+_b[dtime]*dtime)*_b[dtime]*dtime)
summarize dg21 dg22

/*---POE5 Example 16.6---*/
* An Empirical Example from Marketing

clear all
use stata/coke

* examine data
describe
summarize

* linear probability model
regress coke pratio disp_coke disp_pepsi, vce(robust)
estimates store lpm
predict phat

* predict probability when pratio = 1.1
margins, at(pratio=1.1 disp_coke=0 disp_pepsi=0)

* predict outcomes using linear probability model
generate p1 = (phat >=.5)
tabulate p1 coke,row

* probit
probit coke pratio disp_coke disp_pepsi
estimates store probit

* predicted outcomes summary
estat classification

* average marginal effect of change in price ratio
margins, dydx(pratio)

* average marginal effect when pratio=1.1 and displays are not present 
margins, dydx(pratio) at(pratio=1.1 disp_coke=0 disp_pepsi=0)

* average predicted probability when pratio=1.1 and displays are not present
margins, at(pratio=1.1 disp_coke=0 disp_pepsi=0)

* logit
logit coke pratio disp_coke disp_pepsi
estimates store logit

* predicted outcomes summary
estat classification

* average marginal effect of change in price ratio
margins, dydx(pratio)

* average marginal effect: pratio=1.1 & displays not present 
margins, dydx(pratio) at(pratio=1.1 disp_coke=0 disp_pepsi=0)

* average predicted probability: pratio=1.1 & displays not present
margins, at(pratio=1.1 disp_coke=0 disp_pepsi=0)

* tables comparing models
esttab lpm probit logit , se(%12.4f) b(%12.4f) nostar          ///
    scalars(ll_0 ll chi2) gaps mtitles("LPM" "probit" "logit") ///
    title("Coke-Pepsi Choice Models")

/*---POE5 Example 16.7---*/
* Coke Choice Model: Wald Hypothesis Tests

* probit
probit coke pratio disp_coke disp_pepsi
test disp_coke
di "chi-square(1) .05 critical value " invchi2tail(1,.05)
test disp_coke + disp_pepsi=0
lincom disp_coke + disp_pepsi
test disp_coke disp_pepsi
test pratio disp_coke disp_pepsi

/*---POE5 Example 16.8---*/
* Coke Choice Model: Likelihood Ratio Hypothesis Tests

* Test beta3=0
probit coke pratio disp_coke disp_pepsi
estimates store full

* likelihood ratio test of model significance
scalar lnlu = e(ll)

probit coke pratio disp_pepsi
estimates store restricted1
scalar lnlr1 = e(ll)

* automatic test
lrtest full restricted1

scalar lr_test1 = 2*(lnlu-lnlr1)
di "lnlu    = " lnlu 
di "lnlr    = " lnlr1 
di "lr_test = " lr_test1

* likelihood ratio test of displays equal but opposite effect
gen disp = disp_pepsi-disp_coke
probit coke pratio disp
estimates store restricted2

lrtest full restricted2

* likelihood ratio of significance of displays
probit coke pratio
estimates store restricted3

* automatic test
lrtest full restricted3

* model signifiance
probit coke
estimates store restricted4
lrtest full restricted4

probit coke pratio disp_coke disp_pepsi

/*---POE5 Example 16.9---*/
* Estimating the Effect of Education
* on Labor Force Participation

clear all
use stata/mroz
gen nwifeinc = faminc - wage*hours
gen exper2 = exper^2

reg lfp educ exper exper2 kidsl6 age, vce(robust)
reg educ exper exper2 kidsl6 age mothereduc
predict vhat, resid

ivregress 2sls lfp (educ=mothereduc) exper exper2 kidsl6 age, ///
          vce(robust) small

reg lfp educ exper exper2 kidsl6 age vhat, vce(robust)

ivprobit lfp (educ=mothereduc) exper exper2 kidsl6 age

/*---POE5 Example 16.12---*/
* Postsecondary Education Multinomial Choice

clear all
use stata/nels_small
label variable psechoice "post high school choice"
label define psetype 1 "no college" 2 "2-year college" ///
                     3 "4-year college"
label values psechoice psetype

* summarize data
describe
summarize grades, detail
tab psechoice

* estimate model
mlogit psechoice grades, baseoutcome(1)

* compute predictions and summarize
predict ProbNo ProbCC ProbColl
summarize ProbNo ProbCC ProbColl

* predicted probabilities
margins, at(grades=6.64)
margins, at(grades=2.635)

* marginal effects
margins, dydx(grades) at(grades=6.64)
margins, dydx(grades) at(grades=2.635)

* ame
margins, dydx(grades)

/*---POE5 Example 16.13---*/
* Conditional Logit Soft Drink Choice

clear all
use stata/cola

* examine data
describe
summarize
list in 1/9, sepby(id)

* create alternatives variable
sort id, stable
by id:gen alt = _n

* view some observations
list in 1/3

* label values
label define brandlabel 1 "Pepsi"  2  "SevenUp" 3 "Coke" 
label values alt brandlabel

* summarize by alternative
bysort alt:summarize choice price feature display

* estimate model
asclogit choice price, case(id) alternatives(alt) basealternative(Coke)

* post-estimation
estat alternatives
estat mfx
estat mfx, at(Coke:price=1.10 Pepsi:price=1 SevenUp:price=1.25)

/*---POE5 Example 16.14---*/
* Postsecondary Education Ordered Choice Model

clear all
use stata/nels_small
label variable psechoice "post high school choice"
label define psetype 1 "no college" 2 "2-year college" ///
                     3 "4-year college"

label values psechoice psetype
* summarize data
summarize grades, detail
tab psechoice

* estimate model
oprobit psechoice grades

* marginal effects
margins, dydx(grades) at(grades=6.64)
margins, dydx(grades) at(grades=2.635)

/*---POE5 Example 16.15---*/
* A Count Model for the Number of Doctor Visits

clear all
use stata/rwm88_small
describe docvis age female public
summarize docvis age female public
tabulate docvis
poisson docvis age female public

nlcom exp(_b[_cons] + _b[age]*30 + _b[female] + _b[public])*_b[age]
nlcom exp(_b[_cons] + _b[age]*50 + _b[female] + _b[public])*_b[age]
nlcom exp(_b[_cons] + _b[age]*70 + _b[female] + _b[public])*_b[age]

qui poisson docvis age female public
margins, dydx(age) at(age=(30 50 70) female=1 public=1)

nlcom (exp(_b[_cons] + _b[age]*30 + _b[female] + _b[public])-   ///
       exp(_b[_cons] + _b[age]*30 + _b[female]))

nlcom (exp(_b[_cons] + _b[age]*50 + _b[female] + _b[public])-   ///
       exp(_b[_cons] + _b[age]*50 + _b[female]))

nlcom (exp(_b[_cons] + _b[age]*70 + _b[female] + _b[public])-   ///
       exp(_b[_cons] + _b[age]*70 + _b[female]))

qui poisson docvis age i.female i.public
margins, dydx(public) at(age=(30 50 70) female=1) vsquish  

poisson docvis age female public
gen docvishat = exp(_b[_cons] + _b[age]*age + _b[female]*female ///
    + _b[public]*public)

gen dochat = round(docvishat)
list docvis age female public docvishat dochat in 1/10

pwcorr dochat docvis

/*---POE5 Example 16.16---*/
* Tobit Model of Hours Worked

clear all
use stata/mroz

* examine data
describe lfp hours educ exper age kidsl6
summarize lfp hours educ exper age kidsl6, separ(0)

histogram hours, frequency title(Hours worked by married women)

summarize hours  educ exper age kidsl6 if (hours>0)
summarize hours  educ exper age kidsl6 if (hours==0)

* regression
regress hours  educ exper age kidsl6
regress hours  educ exper age kidsl6 if (hours>0)

* tobit
tobit hours  educ exper age kidsl6, ll

* marginal effect on E(y|x)
margins, dydx(educ) atmeans at(kidsl6=1) ///
         predict(ystar(0,.))

* marginal effect on E(y|x,y>0)
margins, dydx(educ) atmeans at(kidsl6=1) ///
         predict(e(0,.))

/*---POE5 Example 16.17---*/
* Heckit Model of Wages

clear all
use stata/mroz

generate lwage = ln(wage)

* ols
regress lwage educ exper if (hours>0)

* probit
generate kids = kidsl6+kids618>0 if !missing(kidsl6,kids618)
probit lfp age educ kids mtr
predict w, xb

* Inverse Mills Ratio
generate imr = normalden(w)/normal(w)

* Heckit two-step
regress lwage educ exper imr

* Heckit two-step automatic
heckman lwage educ exper, select(lfp=age educ kids mtr) twostep

* Heckit maximum likelihood
heckman lwage educ exper, select(lfp=age educ kids mtr)

/*---Appendix 16D---*/
* A Tobit Monte Carlo Experiment

clear all

program tobitmc, rclass
    version 15.1
    drop _all
    syntax, [numobs(int 200) beta1(real -9) beta2(real 1) sigma(real 4)]
    set obs `numobs'
    
    generate double x = runiform(0,20)
    gen ey = `beta1' + `beta2'*x
    gen e = rnormal(0, `sigma')
    gen ystar = ey + e
    gen d = (ystar>0)
    gen y = d*ystar

    * OLS latent variable
    reg ystar x
    return scalar b1 = _b[_cons]
    return scalar b2 = _b[x]
    return scalar se1 = _se[_cons]
    return scalar se2 = _se[x]

    * OLS positive values
    reg y x if y>0
    return scalar b1p = _b[_cons]
    return scalar b2p = _b[x]
    return scalar se1p = _se[_cons]
    return scalar se2p = _se[x]

    * OLS all values
    reg y x
    return scalar b1u = _b[_cons]
    return scalar b2u = _b[x]
    return scalar se1u = _se[_cons]
    return scalar se2u = _se[x]

    * tobit
    tobit y x, ll(0)
    return scalar b1t = _b[_cons]
    return scalar b2t = _b[x]
    return scalar se1t = _se[_cons]
    return scalar se2t = _se[x]	
end

set seed 1234567
tobitmc
saveold tobit5, replace 

reg ystar x
estimates store model1
reg y x if y>0
estimates store model2
reg y x
estimates store model3
tobit y x, ll(0)
estimates store model4

esttab model1 model2 model3 model4 , se(%10.4f)  b(%10.4f)  ///
     nostar nogaps title(POE5 Table 16D.1)                  ///
     mtitles("ystar" "y>0" "y" "Tobit")

simulate b1r = r(b1) b2r = r(b2) se1r = r(se1) se2r = r(se2)    ///
  b1pr = r(b1p) b2pr = r(b2p) se1pr = r(se1p) se2pr = r(se2p)   ///
  b1ur = r(b1u) b2ur = r(b2u) se1ur = r(se1u) se2ur = r(se2u)   ///
  b1tr = r(b1t) b2tr = r(b2t) se1tr = r(se1t) se2tr = r(se2t) , ///
  reps(1000) saving(tobitmc, replace)                       ///
  seed(1234567) nolegend nodots: tobitmc

summarize b1r b2r se1r se2r              // latent data regression
summarize b1pr b2pr se1pr se2pr          // y > 0 regression
summarize b1ur b2ur se1ur se2ur          // all y, x data regression
summarize b1tr b2tr se1tr se2tr          // tobit regression
